/* Finding ordered clusters
 * Author: Qingwu Yang
 * Texas A&M University
 * July 6, 2008
**/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <time.h>
#include <math.h>

#define EXIT0       fflush(NULL), exit(0)
#define FPFE1(x)    fprintf(stdout, "%s", x), fflush(NULL), exit(0)
#define FPFE2(x, y) fprintf(stdout, x, y), fflush(NULL), exit(0)

int  MIN_SET_OUTPUT = 1;     //the minimal size of subsets that will be output
double EVALUE_CUTOFF = 1e-1;  //cutoff for e-values
int  MIN_N = 2;              //the minimal number of chromosomes

int NUM_GENOME_CUTOFF = 2; //the minimum # of genomes that the clusters should be in 

int  IF_CIRCULAR_GENOME = 0; 

int SYNTENY_DEFINITION = 2;  //1, no paralog. no order
                             //2, no paralog, same order

int NUM_OUTPUT = 200; //number of output required for SYNTENY_DEFINITION = 2

int     ave_L;
int     dist_k = 0;      //distance cutoff

int   *ChrNO; //current chr# of each genome

typedef struct ssetindex {
	int  a;
	int  n;
} ssetindex;

typedef struct rnode {
	int   *set;
	double eval;      //evalues  = EVALUE[avl][n]
	int    l;         //# of unique clusters
	int    n;         //# of chromosomes that contain this cluster
//	int    avl;       //average length of start-end distances
//	int    avcl;      //average length of chromosomes
//	char ** s;
//	int    *chr;      //chromosome ID's
	int    *low;      //starting point on the first chromosome (reference window)
	int    *high;     //end point on the first chromosome
	int    *numg;     //# of genes on each chromosome
	int   **posg;     //gene positions on each chromosome
	struct rnode  *next;
} rnode;

	typedef struct EDGE_t {
		int v1;  //node 1
		int v2;  //node 2
		int w;   //weight
	} EDGE_t;

typedef struct bigdouble {
	double d;
	double e;
} bigdouble;


//info()
void info(char * pro);

//parse commandline
void cmd_parser(int ac, char ** av, int * k, char * fname, char *ofname);

//check if a line is blank
int is_blank(char * s);

//get file names containing input data
void get_file_names(char * fname, char *** genomenames, int *num_genomes);

//read in data
void data_reader(int ***gchrs, int **num_genes, char **genomenames, int num_genomes);

//data transform --- gene -- poss
void data_transform(int *** xposs, int *gmax, int n_genomes, int * n_genes, int ** xchrs);

int * tmp_K;

int * tarr, n_tarr;
int ** sarr, * n_sarr;
int  * cstat; //array for fast intersection
int ggmax;
int *tmpx;
int **allcomb, allind, *allindex;

double ** EVALUE;
double ***allEVALUES;

//calculate avervage length
int ave_l(int * n_genes, int n_genomes);

//calculate min length
int min_l(int * n_genes, int n_genomes);

//calculate all e-values -- fast
void calcultate_ALL_EVALUE_FAST(int l, int n);

//calculate e-values between 0 and r 
void comput_ALL_EVALUE(int l, int n, int r);

//calculate all e-values
void comput_order_ALL_EVALUE(int l, int n, int r);

//search for syntenies with zz Method
void syn_finder_zz(int **xchr, int **xpos, char **xname, char **xnomename, int n_genome,
					  int *n_gene, int xmax, int k, char *ofname);

//search for syntenies with genes in same order
void syn_finder_order(int **xchr, int **xpos, char **xname, char **xnomename, int n_genome,
					  int *n_gene, int xmax, int k, char *ofname);

//search for syntenies with genes in same order -- in two directions
void syn_finder_order_2ways(int **xchr, int **xpos, char **xname, char **xnomename, int n_genome,
					  int *n_gene, int xmax, int k, char *ofname);

//search for syntenies with genes in same order -- in two directions
//for circurlar genomes
void syn_finder_order_2ways_circular(int **xchr, int **xpos, char **xname, char **xnomename, int n_genome,
					  int *n_gene, int xmax, int k, char *ofname);


int main(int argc, char ** argv){
	//FILE * outf;
	int  num_genomes, *num_genes, gmax;
	int  ** gchrs;       //genes on all chromosomes. positive int -- cluster ID; -1 -- the gene not counted.
	int  ** gposs;       //gene positions on the genomes. gposs[x][y]: positive int -- position of cluster x on chr y; -1 -- x not on y. 
	char ** gnames;      //gene names
	char ** genomenames; //genome names
	char    datafname[30]; //name of file containing input data
	char    outfname[30];  //name of file containing output
	//char    tmpoutfname[40];

	//int i; //,j; //,k;
	//time_t  time0, time3, time1, time2;

	//time0 = time(NULL);
	
	//if( !(outf = fopen("synlog", "a")) )
	//	FPFE1("can't open synlog for writing");

	//parse commandline
	strcpy(outfname, "syn_result");
	cmd_parser(argc, argv, &dist_k, datafname, outfname);
	//strcpy(tmpoutfname, outfname);
	//strcat(tmpoutfname, "_tmpxyz");

	//get file names containing input data
	get_file_names(datafname, &genomenames, &num_genomes);

	//read in data -- genes on chromosomes in order
	data_reader(&gchrs, &num_genes, genomenames, num_genomes);

	//data transform --- gene -- poss
	data_transform(&gposs, &gmax, num_genomes, num_genes, gchrs);

	//fprintf(outf, "\n");
	//for(i=0; i<num_genomes; i++)
	//	fprintf(outf, " %d", ChrNO[i]);

	//calcualte e-values
	ave_L = ave_l(num_genes, num_genomes);
	//calculate min length
	//ave_L = min_l(num_genes, num_genomes);

	ggmax = gmax;

	//search for syntenies
	gnames = NULL;

	if( SYNTENY_DEFINITION == 1 ){ // no paralog, no order
		//time1 = time(NULL);
		if( dist_k < 0 ){
			fprintf(stderr, "\ndistance Cutoff is negative ?!\n"), fflush(NULL), exit(0);
		} else if (dist_k == 0 || dist_k >= ave_L){
			calcultate_ALL_EVALUE_FAST(ave_L, num_genomes);
		} else {
			comput_ALL_EVALUE(ave_L, num_genomes, dist_k);
		}

		//time2 = time(NULL);
		//fprintf(outf, "\ncalculate_all_EVALUE: %7.2f mins\n", (double)difftime(time2, time1)/60), fflush(NULL);

		//search for syntenies with zz Method
		syn_finder_zz(gchrs, gposs, gnames, genomenames, num_genomes, num_genes, gmax, dist_k, outfname);
	} else if ( SYNTENY_DEFINITION == 2 ){ //  no paralog, same order
		NUM_GENOME_CUTOFF = num_genomes; 

		//time1 = time(NULL);
		if( dist_k < 0 ){
			fprintf(stderr, "\ndistance Cutoff is negative ?!\n"), fflush(NULL), exit(0);
		} 
		/*
		else if (dist_k == 0 || dist_k >= ave_L){
			comput_order_ALL_EVALUE(ave_L, num_genomes, ave_L);
		} else {
			comput_order_ALL_EVALUE(ave_L, num_genomes, dist_k+1);
		}
		*/

		//use same formula as non-order one
		comput_ALL_EVALUE(ave_L, num_genomes, dist_k+1);

		//time2 = time(NULL);
		//fprintf(outf, "\ncalculate_all_EVALUE: %7.2f mins\n", (double)difftime(time2, time1)/60), fflush(NULL);

		//search for syntenies with genes in same order
		//syn_finder_order(gchrs, gposs, gnames, genomenames, num_genomes, num_genes, gmax, dist_k, outfname);

		if( IF_CIRCULAR_GENOME == 1 ){
			//search for syntenies with genes in same order -- in two directions
			//for circurlar genomes
			syn_finder_order_2ways_circular(gchrs, gposs, gnames, genomenames, num_genomes, num_genes, gmax, dist_k, outfname);
		} else {
			//search for syntenies with genes in same order -- in two directions
			syn_finder_order_2ways(gchrs, gposs, gnames, genomenames, num_genomes, num_genes, gmax, dist_k, outfname);
		}
	} else {
		fprintf(stderr, "\n\nWrong SYNTENY_DEFINITION choice: (%d)\n\n", SYNTENY_DEFINITION), fflush(NULL), exit(0);
	}

	//time3 = time(NULL);
	//fprintf(outf, "\nTotal time for this comb: %7.2f mins\n", (double)difftime(time3, time0)/60), fflush(NULL);
	//fclose(outf);
	
	return 0;
} //main()

//calculate avervage length
int ave_l(int * n_genes, int n_genomes){
	int i,j;

	for(i=0, j=0; i<n_genomes; i++)
		j += n_genes[i];

	return (j/n_genomes);
}

//calculate min length
int min_l(int * n_genes, int n_genomes){
	int i,j;

	j = SHRT_MAX;

	for(i=0; i<n_genomes; i++)
		if( n_genes[i] < j )
			j = n_genes[i];

	return j;
}


void free_rnode(rnode * r, int ng){
	int y;
	if( !r )
		return;

	//free space
	if( r->set )
		free(r->set);
	//if( r->chr ) 	free(r->chr);
	if( r->low )
		free(r->low);
	if( r->high )
		free(r->high);
	if( r->numg )
		free(r->numg);
	if( r->posg ){
		for(y=0; y<ng; y++)
			if( r->posg[y] )
				free(r->posg[y]);
		free(r->posg);
	}

	if( r )
		free(r);

	return;
}

//output rnodes
void output_rnodes(int **xchr, char **xnomename, int n_genome, char *ofname, rnode * r){
	FILE * outf;
	int i, j;

	rnode *curr;

	if( ! r->next )
		return;

	if( !(outf = fopen(ofname, "a")) )
		FPFE2("File %s can't be opened for appending\n", ofname);

	curr = r->next;

	while(curr){
		fprintf(outf, "\n\nCluster: ");
		for(i=0; i<curr->l; i++)
			fprintf(outf, " %1d", curr->set[i]);

		fprintf(outf, "\n\tExpect = %10.4e, Size = %1d/%1d, Appear = %1d/%1d",
			           curr->eval, curr->l, ave_L, curr->n, n_genome);
		//*
		for(i=0; i<n_genome; i++){
			if( curr->numg[i] ){
				fprintf(outf, "\n\t%15s (Chr:%1d S%1d E%1d):", xnomename[i], ChrNO[i], curr->low[i], curr->high[i]);
				for(j=0; j<curr->numg[i]; j++)
					fprintf(outf, " %2d (%2d)", xchr[i][curr->posg[i][j]], curr->posg[i][j]);
			} else {
				fprintf(outf, "\n@@");
			}
		}

		curr = curr->next;
	}

	fclose(outf);

	return;
}


//remove repeats from a linked list of rnodes
void remove_repeat(rnode *head, int ng)//, char *tfname, char * ofname){
{
	int   n, yc, yd, y1;

	rnode *curr, *burr, *durr, *turr;

	n = ng + 2;

	 //remove repeats and subsets
	 if( head->next && (head->next)->next ){
		 burr = head->next;
		 curr = (head->next)->next;
		 while( curr ){
			 durr = head->next;
			 while( durr != curr ){
				 if( durr->l >= curr->l && durr->n >= curr->n){
					 yc = yd = 0;
					 while( yc < curr->l && yd < durr->l ){
						 if( curr->set[yc] == durr->set[yd] ){
							 yc++;
							 yd++;
						 } else if ( curr->set[yc] < durr->set[yd] ){
							 break;
						 } else {
							 yd++;
						 }
					 }

					 if( yc == curr->l ){ // possible -- subset or same
						for( y1=0; y1<ng; y1++ ){
							if( curr->numg[y1] >0 ){
								if( !(curr->numg[y1] <= durr->numg[y1] && curr->low[y1]>=durr->low[y1] && curr->high[y1]<=durr->high[y1]) )
									break;
							}
						}
						if( y1 == ng ){ // subset or same
							turr = curr;
							burr->next = curr->next;
							curr = curr->next;

							//free space
							free_rnode(turr, ng);
							
							break;
						}

					 }
				 }

				 durr = durr->next;
			 }

			 if ( durr == curr ) {
				 burr=burr->next;
				 curr=curr->next;
			 }
		 }
	 }

	return;
}


//combine 2 linked lists of rnodes; add r2 to r1
void combine2lists(rnode *r1, rnode * r2, int n) {
	rnode *curr, *burr, *durr;

	durr = r2->next;
	while( durr ){
		r2->next = durr->next;
		durr->next = NULL;

		//add the new node to the linked list
		burr = r1;
		curr = r1->next;

		while( curr && curr->l >= durr->l ){
			burr = burr->next;
			curr = curr->next;
		}

		if( curr == NULL ){
			burr->next = durr;
		} else {
			burr->next = durr;
			durr->next = curr;
		}

		durr = r2->next;
	}

	return;	
}

//combine 2 linked lists of rnodes; add r2 to r1; using evalue as cutoff
void combine2lists_evalcut(rnode *r1, rnode * r2, int n) {
	rnode *curr, *burr, *durr;

	durr = r2->next;
	while( durr ){
		r2->next = durr->next;
		durr->next = NULL;

		if( durr->eval <= EVALUE_CUTOFF ){
			//add the new node to the linked list
			 burr = r1;
			 curr = r1->next;

			 while( curr && curr->eval <= durr->eval ){
				 burr = burr->next;
				 curr = curr->next;
			 }

			 if( !curr ){
				 burr->next = durr;
			 } else {
				 burr->next = durr;
				 durr->next = curr;
			 }
		} else
			free_rnode(durr, n);

		 durr = r2->next;
	}

	return;
}

///////<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

//data transform --- gene -- poss
void data_transform(int *** xposs, int *gmax, int n_genomes, int * n_genes, int ** xchrs){
	int max, i, j;
	int ** pos;

	max = 0;
	for(i=0; i<n_genomes; i++)
		for(j=0; j<=n_genes[i]; j++)
			if( max < xchrs[i][j] )
				max = xchrs[i][j];
	*gmax = max;
	if( !(pos = malloc((max+1) * sizeof(int *))) )
		FPFE1("out of memory for pos\n");
	for(i=0; i<=max; i++){
		if( !(pos[i] = malloc(n_genomes * sizeof(int))) )
			FPFE1("out of memory for pos[]\n");
		for(j=0; j<n_genomes; j++)
			pos[i][j] = -1;
	}

	for(i=0; i<n_genomes; i++){	
		for(j=0; j<=n_genes[i]; j++){
			if( xchrs[i][j] != -1 )
				pos[xchrs[i][j]][i] = j;				
		}		
	}

	*xposs = pos;

	return;
}


//read in data
void data_reader(int ***gchrs, int **num_genes, char **genomenames, int num_genomes){
	FILE *inf;
	int ** xchrs;
	int *  xnum;
	int i,j, n, m, max;
	char buff[4096];

	if( !(xnum = malloc(num_genomes * sizeof(int))) )
		FPFE1("out of memory for xnum\n");
	if( !(xchrs = malloc(num_genomes * sizeof(int *))) )
		FPFE1("out of memory for xchrs\n");

	for(i=0; i<num_genomes; i++){
		if( !(inf = fopen(genomenames[i], "r")) )
			FPFE2("File %1s can't be opened for reading!\n", genomenames[i]);

		max = 0;
		while ( fgets(buff, 4096, inf) ){
			if( !is_blank(buff) ){
				if( sscanf(buff, " %d %d ", &m, &n) != 2 ){
					FPFE2("\nData error in line %1s ", buff);
					FPFE2(" In file %1s\n", genomenames[i]);
				}
				if( max < m )
					max = m;
			}
		}

		xnum[i] = max;
		if( !(xchrs[i] = malloc( (max+1) * sizeof(int) )) )
			FPFE1("out of memory for xchrs[]\n");

		for(j=0; j<=max; j++)
			xchrs[i][j] = -1;

		rewind(inf);
		while ( fgets(buff, 4096, inf) ){
			if( !is_blank(buff) ){
				if( sscanf(buff, " %d %d ", &m, &n) != 2 ){
					FPFE2("\nData error in line %1s ", buff);
					FPFE2(" In file %1s\n", genomenames[i]);
				}
				xchrs[i][m] = n;
			}
		}

		fclose(inf);
	}

	*gchrs= xchrs;
	*num_genes = xnum;

	return;
}

//get file names containing input data
void get_file_names(char * fname, char *** genomenames, int *num_genomes){
	FILE *inf;
	char buff[4096], *ptr;
	char ** gns;
	int n=0;

	if( !(inf = fopen(fname, "r")) )
		FPFE2("File %1s can't be opened for reading", fname);
	while( fgets(buff, 4096, inf) )
		if( !is_blank(buff) )
			n++;

	*num_genomes = n;
	if( !(gns = malloc(n * sizeof(char *))) )
		FPFE1("out of memory for *genomenames!\n");

	rewind(inf);
	n=0;
	while( fgets(buff, 4096, inf) ){
		if( !is_blank(buff) ){
			ptr=buff;
			while( *ptr != '\0' && *ptr != '\n' && *ptr != EOF )
				ptr++;
			if( *ptr == '\n' || *ptr == EOF )
				*ptr = '\0';

			if( !( gns[n] = malloc( (strlen(buff)+1) * sizeof(char))) )
				FPFE1("out of memory for *genomenames[]!\n");
			strcpy(gns[n], buff);
			n++;
		}
	}
	*genomenames = gns;
	fclose(inf);
	return;
}

//check if a line is blank
int is_blank(char * s){
	while( *s != '\0' && *s != EOF ){
		if( isalnum(*s) )
			return 0;
		s++;
	}
	return 1;
}

//info()
void info(char * pro){
	fprintf(stdout, "\n%s -i=inputfile  -f=linear_or_circular_genome  -k=distance_cutoff -n=min_set_to_be_output -e=e_vaule_cutoff -o=outputfile -c=chr#\n\n", pro);
	fflush(NULL);
	exit(0);
}

//parse commandline
void cmd_parser(int ac, char ** av, int * k, char * fname, char *ofname){
	if( ac < 3 ){
		info(av[0]);
	} else {
		ac--;
		while( ac > 0 ){
			if( strstr(av[ac], "-i=") ){
				if( sscanf(av[ac], "-i=%s", fname) != 1 ){
					fprintf(stdout, "wrong command line input: %s\n", av[ac]);
					info(av[0]);
				}
			} else if( strstr(av[ac], "-f=") ){
				if( sscanf(av[ac], "-f=%d", &IF_CIRCULAR_GENOME) != 1 ){
					fprintf(stdout, "wrong command line input: %s\n", av[ac]);
					info(av[0]);
				}
			} else if( strstr(av[ac], "-o=") ){
				if( sscanf(av[ac], "-o=%s", ofname) != 1 ){
					fprintf(stdout, "wrong command line input: %s\n", av[ac]);
					info(av[0]);
				}
			} else if( strstr(av[ac], "-k=") ){
				if( sscanf(av[ac], "-k=%d", k) != 1 ){
					fprintf(stdout, "wrong command line input: %s\n", av[ac]);
					info(av[0]);
				}
			} else if( strstr(av[ac], "-n=") ){
				if( sscanf(av[ac], "-n=%d", &MIN_SET_OUTPUT) != 1 ){
					fprintf(stdout, "wrong command line input: %s\n", av[ac]);
					info(av[0]);
				}
			} else if( strstr(av[ac], "-e=") ){
				if( sscanf(av[ac], "-e=%lf", &EVALUE_CUTOFF) != 1 ){
					fprintf(stdout, "wrong command line input: %s\n", av[ac]);
					info(av[0]);
				}
			} else if( strstr(av[ac], "-c=") ){
				int i,j;
				char *p, *q;
				p=av[ac];
				j=0;
				while( strstr(p, "x") ){
					j++;
					p++;
				}

				if( !(ChrNO = malloc(j * sizeof(int))) )
					FPFE1("out of memory for ChrNO\n");
				p=av[ac];
				p=strstr(p, "x");
				q=strtok(p, "x");
				ChrNO[0] = atoi(q);
				i=1;
				while( (q=strtok(NULL, "x")) ){
					ChrNO[i] = atoi(q);
					i++;
				}
			} else {
				fprintf(stdout, "wrong command line input: %s\n", av[ac]);
				info(av[0]);
			}
			ac--;
		}
	}

	return;
}


//do all combinations recurrsively
void recur_comb(int nGenomes, int n){
	int i, j;

	for(i=1; i>=0; i--){
		tmpx[n] = i;

		if( n == nGenomes-1 ){
			for(j=0; j<nGenomes; j++)
				allcomb[allind][j] = tmpx[j];

			allind ++;
		} else {
			recur_comb(nGenomes, n+1);
		}
	}
}


//pppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppppp
//calculate combination
void comb(bigdouble ** c, int l){
	bigdouble * b; 
	double x;
	int y, k;

	if( !(b= calloc(l+1, sizeof(bigdouble))) )
		FPFE1("out of memory for b");
	
	b[0].d = 1.0;
	b[0].e = 0;
	for(k=1; k<=l/2; k++){
		x = (double)(l-k+1)/k;

		if( x == 0.0 )
			fprintf(stdout, "\nl=%d k=%d", l, k), fflush(NULL), exit(0);

		y = 0;
		if( x >= 10.0 ){
			while(x >= 10.0){
				x /= 10.0;
				y++;
			}
		} else if ( x < 1.0 ) {
			while( x < 1.0){
				x *= 10.0;
				y--;
			}
		}	
		b[k].e = b[k-1].e + y;
		b[k].d = b[k-1].d * x;

		if( b[k].d >= 10.0 ){
			b[k].d /= 10.0;
			b[k].e += 1;
		}
		//b[k] = (double)(l-k+1)/k * b[k-1];
		//if( l == 2 )
		//	fprintf(stdout, "\nb[%3d]= %2.5e, %2.5e", k, b[k].d, b[k].e);
	}

	for(k=l/2+1; k<=l; k++){
		b[k] = b[l-k];
		//fprintf(stdout, "\nb[%3d]= %2.5e", k, b[k]);
		//if( l == 2 )
		//	fprintf(stdout, "\nb[%3d]= %2.5e, %2.5e", k, b[k].d, b[k].e);
	}

	*c = b;

	return;
}

//calculate 2-d combination -- k, k'
void comb2d(bigdouble *** c, int l){
	bigdouble **d;

	if( !(d = (bigdouble **)calloc(l+1, sizeof(bigdouble *))) )
		FPFE1("out of memory for d");

	while(l >= 0){
		comb(d+l, l);
		l--;
	}

	*c = d;
	return;
}

bigdouble bigdouble_mult(bigdouble a, bigdouble b){
	bigdouble c;
	double x;
	int y;

	if( a.d == 0.0 || b.d == 0.0 ){
		c.d = 0.0;
		c.e = 0.0;
	} else {
		c.e = a.e + b.e;
		x = a.d * b.d;

		y = 0;
		if( x >= 10.0 ){
			while(x >= 10.0){
				x /= 10.0;
				y++;
			}
		} else if ( x < 1.0 ) {
			if( x == 0.0 )
				fprintf(stdout, "\nHerem x =0 "), fflush(NULL), exit(0);
			if( x < 0.0 ){
				fprintf(stdout, "\na.d = %f, .e = %e",a.d, a.e);
				fprintf(stdout, "\nb.d = %f, .e = %e",b.d, b.e);
				fprintf(stdout, "\nHerem x < 0 "), fflush(NULL), exit(0);
			}

			while( x < 1.0){
				x *= 10.0;
				y--;
			}
		}

		c.e += y;
		c.d  = x;
	}

	return c;	
}

//calculate p(l, k, k')
void cal_pLKKprime(bigdouble **b, bigdouble ****p, bigdouble ****p1, int l, int n){
	bigdouble ***d;
	bigdouble ***e;
	int i, j, k;
	double x, y;

	if( !(d = (bigdouble ***)calloc(n, sizeof(bigdouble **))) )
		FPFE1("out of memory for d");
	for(i=0; i<n; i++){
		if( !(d[i] = (bigdouble **)calloc(l, sizeof(bigdouble *))) )
			FPFE1("out of memory for d[]");

		for(j=0; j<l; j++)
			if( !(d[i][j] = (bigdouble *)calloc(j+1, sizeof(bigdouble))) )
				FPFE1("out of memory for d[][]");
	}

	if( !(e = (bigdouble ***)calloc(n, sizeof(bigdouble **))) )
		FPFE1("out of memory for e");
	for(i=0; i<n; i++){
		if( !(e[i] = (bigdouble **)calloc(l, sizeof(bigdouble *))) )
			FPFE1("out of memory for e[]");

		for(j=0; j<l; j++)
			if( !(e[i][j] = (bigdouble *)calloc(j+1, sizeof(bigdouble))) )
				FPFE1("out of memory for e[][]");
	}

	//d
	for(i=0; i<l; i++){
	//for(i=1; i<2;i++){
		for(j=0; j<i+1; j++){
			x = (double)(l-i-1);
			//if( x == 0.0 ){
			//	d[0][i][j].d = 0.0;
			//	d[0][i][j].e = 0;
			//} else {
				y=0;
				while(x >= 10.0){
					x /= 10.0;
					y += 1;
				}

				d[0][i][j].d = x * b[i][j].d;
				d[0][i][j].e = y + b[i][j].e;

				if( d[0][i][j].d >= 10.0 ){
					d[0][i][j].d /= 10.0;
					d[0][i][j].e ++;
				}
			//}

			//fprintf(stdout, "\nb[%d][%d].e = %f, .d = %e", i, j, b[i][j].e, b[i][j].d);
			//fprintf(stdout, "\nd[0][%d][%d].e = %f, .d = %e", i, j, d[0][i][j].e, d[0][i][j].d);

			if( d[0][i][j].e >= b[i+1][j+1].e ){				
				y = d[0][i][j].e - b[i+1][j+1].e;
				if( y <= 300 ){				
					d[0][i][j].d += b[i+1][j+1].d / pow(10.0, y);
				}
			} else {
				d[0][i][j].e = b[i+1][j+1].e;
				y = b[i+1][j+1].e - d[0][i][j].e;
				if( y <= 300 ){				
					d[0][i][j].d = b[i+1][j+1].d + d[0][i][j].d / pow(10.0, y);
				}else{
					d[0][i][j].d = b[i+1][j+1].d;
				}
			}

			if( d[0][i][j].d >= 10.0 ){
				d[0][i][j].d /= 10.0;
				d[0][i][j].e  += 1;
			}

			//fprintf(stdout, "\nb[%d][%d].e = %f, .d = %e", i+1, j+1, b[i+1][j+1].e, b[i+1][j+1].d);
			//fprintf(stdout, "\nd[0][%d][%d].e = %f, .d = %e", i, j, d[0][i][j].e, d[0][i][j].d);

			
			x = d[0][i][j].d / b[l][j+1].d;
			y = 0;
			if( x >= 10.0 ){
				while(x >= 10.0){
					x /= 10.0;
					y++;
				}
			} else if ( x < 1.0 ) {
				if( x == 0.0 )
					fprintf(stdout, "\nHere2 x = 0\n"), fflush(NULL), exit(0);
				while( x < 1.0){
					x *= 10.0;
					y--;
				}
			}
			d[0][i][j].e -= b[l][j+1].e;
			d[0][i][j].e += y;	
			d[0][i][j].d  = x;

			//fprintf(stdout, "\nb[%d][%d].e = %f, .d = %e", l, j+1, b[l][j+1].e, b[l][j+1].d);
			//fprintf(stdout, "\nd[0][%d][%d].e = %f, .d = %e", i, j, d[0][i][j].e, d[0][i][j].d);

			//d[0][i][j] = ((l-i-1)*b[i][j] + b[i+1][j+1])/b[l][j+1];
		}
	}

	for(k=1; k<n; k++){
		for(i=0; i<l; i++){
			for(j=0; j<i+1; j++){
				d[k][i][j] = bigdouble_mult(d[k-1][i][j], d[0][i][j]);
			}
		}
	}

	//e
	/*
	for(i=0; i<l; i++){
		for(j=0; j<i+1; j++){
			e[0][i][j] = 1;
		}
	}
	*/
	for(i=0; i<l; i++){
		for(j=0; j<i+1; j++){
			e[1][i][j].e = 0;			
			if( d[0][i][j].e <= -300 ){				
				e[1][i][j].d = 1.0;
			} else {
				e[1][i][j].d = fabs(1.0- pow(10.0, d[0][i][j].e)*d[0][i][j].d);

				if( e[1][i][j].d < 0.0 ){
					fprintf(stdout, "\ni=%d j=%d e.d=%f  b.d = %f, b.e = %e",i,j, e[1][i][j].d, d[0][i][j].d, d[0][i][j].e);
					fprintf(stdout, "\nxerem x < 0 "), fflush(NULL), exit(0);
				}
			}
			//e[1][i][j] = 1-d[0][i][j];
		}
	}

	for(k=2; k<n; k++){
		for(i=0; i<l; i++){
			for(j=0; j<i+1; j++){
				e[k][i][j] = bigdouble_mult(e[k-1][i][j], e[1][i][j]);
			}
		}
	}

	*p = d;
	*p1= e;
	return;
}


//calculate p(n, n', l, k, k')
void cal_pKKN(bigdouble ****e, bigdouble **b, bigdouble ***p, bigdouble ***p1, int l, int n){
	bigdouble ***d;
	int i, j, k;
	double y;

	if( !(d = (bigdouble ***)calloc(l, sizeof(bigdouble **))) )
		FPFE1("out of memory for d");
	for(i=0; i<l; i++){
		if( !(d[i] = (bigdouble **)calloc(i+2, sizeof(bigdouble *))) )
			FPFE1("out of memory for d[]");

		for(j=1; j<i+2; j++)
			if( !(d[i][j] = (bigdouble *)calloc(n+1, sizeof(bigdouble))) )
				FPFE1("out of memory for d[][]");
	}

	for(i=0; i<l; i++){
		for(j=1; j<i+2; j++){			
			d[i][j][n].d = p[n-1][i][j-1].d;
			d[i][j][n].e = p[n-1][i][j-1].e;
		}
	}

	for(i=0; i<l; i++){
		for(k=n-1; k>=1; k--){
			for(j=1; j<i+2; j++){			
				d[i][j][k] = bigdouble_mult(bigdouble_mult(b[n][k],p[k-1][i][j-1]), p1[n-k][i][j-1]);
			}
		}
	}

	for(i=0; i<l; i++){
		for(k=n-1; k>=1; k--){
			for(j=1; j<i+2; j++){
				if( d[i][j][k].e >= d[i][j][k+1].e ){				
					y = d[i][j][k].e - d[i][j][k+1].e;
					if( y <= 300 ){				
						d[i][j][k].d += d[i][j][k+1].d / pow(10.0, y);
					}
				} else {
					d[i][j][k].e = d[i][j][k+1].e;
					y = d[i][j][k+1].e - d[i][j][k].e;
					if( y <= 300 ){										
						d[i][j][k].d = d[i][j][k+1].d + d[i][j][k].d / pow(10.0, y);
					}else{
						d[i][j][k].d = d[i][j][k+1].d;
					}
				}

				if( d[i][j][k].d >= 10.0 ){
					d[i][j][k].d /= 10.0;
					d[i][j][k].e ++;
				}

				//d[i][j][k] += d[i][j][k+1];
			}
		}
	}

	for(i=0; i<l; i++){
		for(j=1; j<i+2; j++){			
			for(k=1; k<=n; k++){
				d[i][j][k] = bigdouble_mult(d[i][j][k], b[l][j]);
			}
		}
	}

	*e = d;
	return;
}


//get evalues in simple format
void cal_EVALUES(bigdouble ***d, int l, int n){
	int i, j, k;

	if( !(allEVALUES = (double ***)calloc(l, sizeof(double **))) )
		FPFE1("out of memory for allEVALUES");
	for(i=0; i<l; i++){
		if( !(allEVALUES[i] = (double **)calloc(i+2, sizeof(double *))) )
			FPFE1("out of memory for allEVALUES[]");

		for(j=1; j<i+2; j++)
			if( !(allEVALUES[i][j] = (double *)calloc(n+1, sizeof(double))) )
				FPFE1("out of memory for allEVALUES[][]");
	}

	for(i=0; i<l; i++){
		for(j=1; j<i+2; j++){			
			for(k=1; k<=n; k++){
				if( d[i][j][k].e >= 1 ){
					allEVALUES[i][j][k] = 10.0;
				} else if ( d[i][j][k].e <= -200 ) {
					allEVALUES[i][j][k] = 0.0;
				} else {
					allEVALUES[i][j][k] = pow(10.0, d[i][j][k].e)*d[i][j][k].d;
				} 
			}
		}
	}

	return;
}

//calculate all e-values
void calcultate_ALL_EVALUE_FAST(int l, int n){
//	FILE *outf;
	int i,j; //, k;
	bigdouble ** comKKprime;
	bigdouble *** pLKKprime; //pLKKprime[0], p(k, k')^1
	bigdouble *** pLKKprime_1;
	bigdouble *** pKKN;  //pKKprimeNprime

//	if( !(outf = fopen("testeval", "w")))
//		FPFE1("Can't open file for writing!");

	//calculate 2-d combination -- k, k'
	comb2d(&comKKprime, l);

	//calculate p(l, k, k') and powers
	cal_pLKKprime(comKKprime, &pLKKprime, &pLKKprime_1, l, n);

	//calculate e values
	cal_pKKN(&pKKN, comKKprime, pLKKprime, pLKKprime_1, l, n);

	//get evalues in simple format
	cal_EVALUES(pKKN, l, n);

	/*
	for(i=0; i<l; i++){
		fprintf(outf, "\n\n>l= %d", i+1);
		for(j=1; j<=i+1; j++){
			fprintf(outf, "\n%4d", j);
			for(k=1; k<=n; k++){
				fprintf(outf, " %2.5f e%4d", pKKN[i][j][k].d, (int)pKKN[i][j][k].e);
			}
		}
	}
	fclose(outf);
	*/

	//Free space
	if( comKKprime ){
		for(i=0; i<=l; i++)
			if( comKKprime[i] )
				free(comKKprime[i]);
		free(comKKprime);
	}

	if( pLKKprime ){
		for(i=0; i<n; i++){
			for(j=0; j<l; j++){
				if( pLKKprime[i][j] )
					free( pLKKprime[i][j] );
			}
			free( pLKKprime[i] );
		}
		free( pLKKprime );
	}

	if( pLKKprime_1 ){
		for(i=0; i<n; i++){
			for(j=0; j<l; j++){
				if( pLKKprime_1[i][j] )
					free( pLKKprime_1[i][j] );
			}
			free( pLKKprime_1[i] );
		}
		free( pLKKprime_1 );
	}

	if( pKKN ){
		for(i=0; i<l; i++){
			for(j=1; j<i+2; j++)
				if( pKKN[i][j] )
					free( pKKN[i][j] );
			free( pKKN[i] );
		}
		free( pKKN );
	}

	return;
}


//calculate p(l, k, k')
void comput_pLKKprime(bigdouble **b, bigdouble ****p, bigdouble ****p1, int l, int n, int r){
	bigdouble ***d;
	bigdouble ***e;
	int i, j, k;
	double x, y;

	if( !(d = (bigdouble ***)calloc(n, sizeof(bigdouble **))) )
		FPFE1("out of memory for d");
	for(i=0; i<n; i++){
		if( !(d[i] = (bigdouble **)calloc(r, sizeof(bigdouble *))) )
			FPFE1("out of memory for d[]");

		for(j=0; j<r; j++)
			if( !(d[i][j] = (bigdouble *)calloc(j+1, sizeof(bigdouble))) )
				FPFE1("out of memory for d[][]");
	}

	if( !(e = (bigdouble ***)calloc(n, sizeof(bigdouble **))) )
		FPFE1("out of memory for e");
	for(i=0; i<n; i++){
		if( !(e[i] = (bigdouble **)calloc(r, sizeof(bigdouble *))) )
			FPFE1("out of memory for e[]");

		for(j=0; j<r; j++)
			if( !(e[i][j] = (bigdouble *)calloc(j+1, sizeof(bigdouble))) )
				FPFE1("out of memory for e[][]");
	}

	//d
	for(i=0; i<r; i++){
		for(j=0; j<i+1; j++){
			x = (double)(l-i-1);
			y=0;
			while(x >= 10.0){
				x /= 10.0;
				y += 1;
			}

			d[0][i][j].d = x * b[i][j].d;
			d[0][i][j].e = y + b[i][j].e;

			if( d[0][i][j].d >= 10.0 ){
				d[0][i][j].d /= 10.0;
				d[0][i][j].e ++;
			}

			if( d[0][i][j].e >= b[i+1][j+1].e ){				
				y = d[0][i][j].e - b[i+1][j+1].e;
				if( y <= 300 ){				
					d[0][i][j].d += b[i+1][j+1].d / pow(10.0, y);
				}
			} else {
				d[0][i][j].e = b[i+1][j+1].e;
				y = b[i+1][j+1].e - d[0][i][j].e;
				if( y <= 300 ){				
					d[0][i][j].d = b[i+1][j+1].d + d[0][i][j].d / pow(10.0, y);
				}else{
					d[0][i][j].d = b[i+1][j+1].d;
				}
			}

			if( d[0][i][j].d >= 10.0 ){
				d[0][i][j].d /= 10.0;
				d[0][i][j].e  += 1;
			}
			
			x = d[0][i][j].d / b[l][j+1].d;
			y = 0;
			if( x >= 10.0 ){
				while(x >= 10.0){
					x /= 10.0;
					y++;
				}
			} else if ( x < 1.0 ) {
				if( x == 0.0 )
					fprintf(stdout, "\nHere2 x = 0\n"), fflush(NULL), exit(0);
				while( x < 1.0){
					x *= 10.0;
					y--;
				}
			}
			d[0][i][j].e -= b[l][j+1].e;
			d[0][i][j].e += y;	
			d[0][i][j].d  = x;
		}
	}

	for(k=1; k<n; k++){
		for(i=0; i<r; i++){
			for(j=0; j<i+1; j++){
				d[k][i][j] = bigdouble_mult(d[k-1][i][j], d[0][i][j]);
			}
		}
	}

	//e
	for(i=0; i<r; i++){
		for(j=0; j<i+1; j++){
			e[1][i][j].e = 0;			
			if( d[0][i][j].e <= -300 ){				
				e[1][i][j].d = 1.0;
			} else {
				e[1][i][j].d = fabs(1.0- pow(10.0, d[0][i][j].e)*d[0][i][j].d);

				if( e[1][i][j].d < 0.0 ){
					fprintf(stdout, "\ni=%d j=%d e.d=%f  b.d = %f, b.e = %e",i,j, e[1][i][j].d, d[0][i][j].d, d[0][i][j].e);
					fprintf(stdout, "\nxerem x < 0 "), fflush(NULL), exit(0);
				}
			}
		}
	}

	for(k=2; k<n; k++){
		for(i=0; i<r; i++){
			for(j=0; j<i+1; j++){
				e[k][i][j] = bigdouble_mult(e[k-1][i][j], e[1][i][j]);
			}
		}
	}

	*p = d;
	*p1= e;
	return;
}


//calculate p(l, k, k')   --- for circular chromosomes
void comput_pLKKprime_circular(bigdouble **b, bigdouble ****p, bigdouble ****p1, int l, int n, int r){
	bigdouble ***d;
	bigdouble ***e;
	int i, j, k;
	double x, y;

	if( !(d = (bigdouble ***)calloc(n, sizeof(bigdouble **))) )
		FPFE1("out of memory for d");
	for(i=0; i<n; i++){
		if( !(d[i] = (bigdouble **)calloc(r, sizeof(bigdouble *))) )
			FPFE1("out of memory for d[]");

		for(j=0; j<r; j++)
			if( !(d[i][j] = (bigdouble *)calloc(j+1, sizeof(bigdouble))) )
				FPFE1("out of memory for d[][]");
	}

	if( !(e = (bigdouble ***)calloc(n, sizeof(bigdouble **))) )
		FPFE1("out of memory for e");
	for(i=0; i<n; i++){
		if( !(e[i] = (bigdouble **)calloc(r, sizeof(bigdouble *))) )
			FPFE1("out of memory for e[]");

		for(j=0; j<r; j++)
			if( !(e[i][j] = (bigdouble *)calloc(j+1, sizeof(bigdouble))) )
				FPFE1("out of memory for e[][]");
	}

	//d
	for(i=0; i<r; i++){
		for(j=0; j<i+1; j++){
			//x = (double)(l-i-1);
			x = (double)l;
			y=0;
			while(x >= 10.0){
				x /= 10.0;
				y += 1;
			}

			d[0][i][j].d = x * b[i][j].d;
			d[0][i][j].e = y + b[i][j].e;

			if( d[0][i][j].d >= 10.0 ){
				d[0][i][j].d /= 10.0;
				d[0][i][j].e ++;
			}

			/*
			if( d[0][i][j].e >= b[i+1][j+1].e ){				
				y = d[0][i][j].e - b[i+1][j+1].e;
				if( y <= 300 ){				
					d[0][i][j].d += b[i+1][j+1].d / pow(10.0, y);
				}
			} else {
				d[0][i][j].e = b[i+1][j+1].e;
				y = b[i+1][j+1].e - d[0][i][j].e;
				if( y <= 300 ){				
					d[0][i][j].d = b[i+1][j+1].d + d[0][i][j].d / pow(10.0, y);
				}else{
					d[0][i][j].d = b[i+1][j+1].d;
				}
			}

			if( d[0][i][j].d >= 10.0 ){
				d[0][i][j].d /= 10.0;
				d[0][i][j].e  += 1;
			}
			*/
			
			x = d[0][i][j].d / b[l][j+1].d;
			y = 0;
			if( x >= 10.0 ){
				while(x >= 10.0){
					x /= 10.0;
					y++;
				}
			} else if ( x < 1.0 ) {
				if( x == 0.0 )
					fprintf(stdout, "\nHere2 x = 0\n"), fflush(NULL), exit(0);
				while( x < 1.0){
					x *= 10.0;
					y--;
				}
			}
			d[0][i][j].e -= b[l][j+1].e;
			d[0][i][j].e += y;	
			d[0][i][j].d  = x;
		}
	}

	for(k=1; k<n; k++){
		for(i=0; i<r; i++){
			for(j=0; j<i+1; j++){
				d[k][i][j] = bigdouble_mult(d[k-1][i][j], d[0][i][j]);
			}
		}
	}

	//e
	for(i=0; i<r; i++){
		for(j=0; j<i+1; j++){
			e[1][i][j].e = 0;			
			if( d[0][i][j].e <= -300 ){				
				e[1][i][j].d = 1.0;
			} else {
				e[1][i][j].d = fabs(1.0- pow(10.0, d[0][i][j].e)*d[0][i][j].d);

				if( e[1][i][j].d < 0.0 ){
					fprintf(stdout, "\ni=%d j=%d e.d=%f  b.d = %f, b.e = %e",i,j, e[1][i][j].d, d[0][i][j].d, d[0][i][j].e);
					fprintf(stdout, "\nxerem x < 0 "), fflush(NULL), exit(0);
				}
			}
		}
	}

	for(k=2; k<n; k++){
		for(i=0; i<r; i++){
			for(j=0; j<i+1; j++){
				e[k][i][j] = bigdouble_mult(e[k-1][i][j], e[1][i][j]);
			}
		}
	}

	*p = d;
	*p1= e;
	return;
}


//calculate p(n, n', l, k, k')
void comput_pKKN(bigdouble ****e, bigdouble **b, bigdouble ***p, bigdouble ***p1, int l, int n, int r){
	bigdouble ***d;
	int i, j, k;
	double y;

	if( !(d = (bigdouble ***)calloc(r, sizeof(bigdouble **))) )
		FPFE1("out of memory for d");
	for(i=0; i<r; i++){
		if( !(d[i] = (bigdouble **)calloc(i+2, sizeof(bigdouble *))) )
			FPFE1("out of memory for d[]");

		for(j=1; j<i+2; j++)
			if( !(d[i][j] = (bigdouble *)calloc(n+1, sizeof(bigdouble))) )
				FPFE1("out of memory for d[][]");
	}

	for(i=0; i<r; i++){
		for(j=1; j<i+2; j++){			
			d[i][j][n].d = p[n-1][i][j-1].d;
			d[i][j][n].e = p[n-1][i][j-1].e;
		}
	}

	for(i=0; i<r; i++){
		for(k=n-1; k>=1; k--){
			for(j=1; j<i+2; j++){			
				d[i][j][k] = bigdouble_mult(bigdouble_mult(b[n][k],p[k-1][i][j-1]), p1[n-k][i][j-1]);
			}
		}
	}

	for(i=0; i<r; i++){
		for(k=n-1; k>=1; k--){
			for(j=1; j<i+2; j++){
				if( d[i][j][k].e >= d[i][j][k+1].e ){				
					y = d[i][j][k].e - d[i][j][k+1].e;
					if( y <= 300 ){				
						d[i][j][k].d += d[i][j][k+1].d / pow(10.0, y);
					}
				} else {
					d[i][j][k].e = d[i][j][k+1].e;
					y = d[i][j][k+1].e - d[i][j][k].e;
					if( y <= 300 ){										
						d[i][j][k].d = d[i][j][k+1].d + d[i][j][k].d / pow(10.0, y);
					}else{
						d[i][j][k].d = d[i][j][k+1].d;
					}
				}

				if( d[i][j][k].d >= 10.0 ){
					d[i][j][k].d /= 10.0;
					d[i][j][k].e ++;
				}

				//d[i][j][k] += d[i][j][k+1];
			}
		}
	}

	for(i=0; i<r; i++){
		for(j=1; j<i+2; j++){			
			for(k=1; k<=n; k++){
				d[i][j][k] = bigdouble_mult(d[i][j][k], b[l][j]);
			}
		}
	}

	*e = d;
	return;
}


//get evalues in simple format
void comput_EVALUES(bigdouble ***d, int l, int n, int r){
	int i, j, k;

	if( !(allEVALUES = (double ***)calloc(r, sizeof(double **))) )
		FPFE1("out of memory for allEVALUES");
	for(i=0; i<r; i++){
		if( !(allEVALUES[i] = (double **)calloc(i+2, sizeof(double *))) )
			FPFE1("out of memory for allEVALUES[]");

		for(j=1; j<i+2; j++)
			if( !(allEVALUES[i][j] = (double *)calloc(n+1, sizeof(double))) )
				FPFE1("out of memory for allEVALUES[][]");
	}

	for(i=0; i<r; i++){
		for(j=1; j<i+2; j++){			
			for(k=1; k<=n; k++){
				if( d[i][j][k].e >= 300 ){
					allEVALUES[i][j][k] = 1e300;
				} else if ( d[i][j][k].e <= -300 ) {
					allEVALUES[i][j][k] = 0.0;
				} else {
					allEVALUES[i][j][k] = pow(10.0, d[i][j][k].e)*d[i][j][k].d;
				} 
			}
		}
	}

	return;
}


//calculate all e-values
void comput_ALL_EVALUE(int l, int n, int r){
	int i,j;
	bigdouble ** comKKprime;
	bigdouble *** pLKKprime; //pLKKprime[0], p(k, k')^1
	bigdouble *** pLKKprime_1;
	bigdouble *** pKKN;  //pKKprimeNprime

	//calculate 2-d combination -- k, k'
	comb2d(&comKKprime, l);

	//calculate p(l, k, k') and powers
	//comput_pLKKprime(comKKprime, &pLKKprime, &pLKKprime_1, l, n, r);

	//calculate p(l, k, k')   --- for circular chromosomes
	comput_pLKKprime_circular(comKKprime, &pLKKprime, &pLKKprime_1, l, n, r);
/*
	if ( IF_CIRCULAR_GENOME == 1 ){
		//calculate p(l, k, k')   --- for circular chromosomes
		comput_pLKKprime_circular(comKKprime, &pLKKprime, &pLKKprime_1, l, n, r);
	} else {
		//calculate p(l, k, k') and powers  --- for linear chromosomes
		comput_pLKKprime(comKKprime, &pLKKprime, &pLKKprime_1, l, n, r);
	}
*/
	//calculate e values
	comput_pKKN(&pKKN, comKKprime, pLKKprime, pLKKprime_1, l, n, r);

	//get evalues in simple format
	comput_EVALUES(pKKN, l, n, r);


	if( comKKprime ){
		for(i=0; i<=l; i++)
			if( comKKprime[i] )
				free(comKKprime[i]);
		free(comKKprime);
	}

	if( pLKKprime ){
		for(i=0; i<n; i++){
			for(j=0; j<r; j++){
				if( pLKKprime[i][j] )
					free( pLKKprime[i][j] );
			}
			free( pLKKprime[i] );
		}
		free( pLKKprime );
	}

	if( pLKKprime_1 ){
		for(i=0; i<n; i++){
			for(j=0; j<r; j++){
				if( pLKKprime_1[i][j] )
					free( pLKKprime_1[i][j] );
			}
			free( pLKKprime_1[i] );
		}
		free( pLKKprime_1 );
	}

	if( pKKN ){
		for(i=0; i<r; i++){
			for(j=1; j<i+2; j++)
				if( pKKN[i][j] )
					free( pKKN[i][j] );
			free( pKKN[i] );
		}
		free( pKKN );
	}

	return;
}


//calculate permutation
void permut(bigdouble ** c, int l, int r){
	bigdouble * b; 
	double x;
	int y, k;

	if( !(b= calloc(r+1, sizeof(bigdouble))) )
		FPFE1("out of memory for b");
	
	b[0].d = 1.0;
	b[0].e = 0;
	for(k=1; k<=r; k++){
		x = (double)(l-k+1);

		if( x == 0.0 )
			fprintf(stdout, "\nl=%d k=%d", l, k), fflush(NULL), exit(0);

		y = 0;
		if( x >= 10.0 ){
			while(x >= 10.0){
				x /= 10.0;
				y++;
			}
		} else if ( x < 1.0 ) {
			while( x < 1.0){
				x *= 10.0;
				y--;
			}
		}	
		b[k].e = b[k-1].e + y;
		b[k].d = b[k-1].d * x;

		if( b[k].d >= 10.0 ){
			b[k].d /= 10.0;
			b[k].e += 1;
		}
	}
	
	*c = b;

	return;
}

//calculate p(l, k, k')
void comput_order_pLKKprime(bigdouble **b, bigdouble ****p, bigdouble ****p1, int l, int n, int r, bigdouble * perm){
	bigdouble ***d;
	bigdouble ***e;
	int i, j, k;
	double x, y;

	if( !(d = (bigdouble ***)calloc(n, sizeof(bigdouble **))) )
		FPFE1("out of memory for d");
	for(i=0; i<n; i++){
		if( !(d[i] = (bigdouble **)calloc(r, sizeof(bigdouble *))) )
			FPFE1("out of memory for d[]");

		for(j=0; j<r; j++)
			if( !(d[i][j] = (bigdouble *)calloc(j+1, sizeof(bigdouble))) )
				FPFE1("out of memory for d[][]");
	}

	if( !(e = (bigdouble ***)calloc(n, sizeof(bigdouble **))) )
		FPFE1("out of memory for e");
	for(i=0; i<n; i++){
		if( !(e[i] = (bigdouble **)calloc(r, sizeof(bigdouble *))) )
			FPFE1("out of memory for e[]");

		for(j=0; j<r; j++)
			if( !(e[i][j] = (bigdouble *)calloc(j+1, sizeof(bigdouble))) )
				FPFE1("out of memory for e[][]");
	}

	//d
	for(i=0; i<r; i++){
		for(j=0; j<i+1; j++){
			x = (double)(l-i-1);
			y=0;
			while(x >= 10.0){
				x /= 10.0;
				y += 1;
			}

			d[0][i][j].d = x * b[i][j].d;
			d[0][i][j].e = y + b[i][j].e;

			if( d[0][i][j].d >= 10.0 ){
				d[0][i][j].d /= 10.0;
				d[0][i][j].e ++;
			}

			if( d[0][i][j].e >= b[i+1][j+1].e ){				
				y = d[0][i][j].e - b[i+1][j+1].e;
				if( y <= 300 ){				
					d[0][i][j].d += b[i+1][j+1].d / pow(10.0, y);
				}
			} else {
				d[0][i][j].e = b[i+1][j+1].e;
				y = b[i+1][j+1].e - d[0][i][j].e;
				if( y <= 300 ){				
					d[0][i][j].d = b[i+1][j+1].d + d[0][i][j].d / pow(10.0, y);
				}else{
					d[0][i][j].d = b[i+1][j+1].d;
				}
			}

			if( d[0][i][j].d >= 10.0 ){
				d[0][i][j].d /= 10.0;
				d[0][i][j].e  += 1;
			}
			
			/* //noordered
			x = d[0][i][j].d / b[l][j+1].d;
			y = 0;
			if( x >= 10.0 ){
				while(x >= 10.0){
					x /= 10.0;
					y++;
				}
			} else if ( x < 1.0 ) {
				if( x == 0.0 )
					fprintf(stdout, "\nHere2 x = 0\n"), fflush(NULL), exit(0);
				while( x < 1.0){
					x *= 10.0;
					y--;
				}
			}
			d[0][i][j].e -= b[l][j+1].e;
			d[0][i][j].e += y;	
			d[0][i][j].d  = x;
			*/

			//ordered
			x = d[0][i][j].d / perm[j+1].d;
			y = 0;
			if( x >= 10.0 ){
				while(x >= 10.0){
					x /= 10.0;
					y++;
				}
			} else if ( x < 1.0 ) {
				if( x == 0.0 )
					fprintf(stdout, "\nHere2 x = 0\n"), fflush(NULL), exit(0);
				while( x < 1.0){
					x *= 10.0;
					y--;
				}
			}
			d[0][i][j].e -= perm[j+1].e;
			d[0][i][j].e += y;	
			d[0][i][j].d  = x;			
		}
	}

	for(k=1; k<n; k++){
		for(i=0; i<r; i++){
			for(j=0; j<i+1; j++){
				d[k][i][j] = bigdouble_mult(d[k-1][i][j], d[0][i][j]);
			}
		}
	}

	//e
	for(i=0; i<r; i++){
		for(j=0; j<i+1; j++){
			e[1][i][j].e = 0;			
			if( d[0][i][j].e <= -300 ){				
				e[1][i][j].d = 1.0;
			} else {
				e[1][i][j].d = fabs(1.0- pow(10.0, d[0][i][j].e)*d[0][i][j].d);

				if( e[1][i][j].d < 0.0 ){
					fprintf(stdout, "\ni=%d j=%d e.d=%f  b.d = %f, b.e = %e",i,j, e[1][i][j].d, d[0][i][j].d, d[0][i][j].e);
					fprintf(stdout, "\nxerem x < 0 "), fflush(NULL), exit(0);
				}
			}
		}
	}

	for(k=2; k<n; k++){
		for(i=0; i<r; i++){
			for(j=0; j<i+1; j++){
				e[k][i][j] = bigdouble_mult(e[k-1][i][j], e[1][i][j]);
			}
		}
	}

	*p = d;
	*p1= e;
	return;
}


//calculate p(n, n', l, k, k')
void comput_order_pKKN(bigdouble ****e, bigdouble **b, bigdouble ***p, bigdouble ***p1, int l, int n, int r, bigdouble * perm){
	bigdouble ***d;
	int i, j, k;
	double y;

	if( !(d = (bigdouble ***)calloc(r, sizeof(bigdouble **))) )
		FPFE1("out of memory for d");
	for(i=0; i<r; i++){
		if( !(d[i] = (bigdouble **)calloc(i+2, sizeof(bigdouble *))) )
			FPFE1("out of memory for d[]");

		for(j=1; j<i+2; j++)
			if( !(d[i][j] = (bigdouble *)calloc(n+1, sizeof(bigdouble))) )
				FPFE1("out of memory for d[][]");
	}

	for(i=0; i<r; i++){
		for(j=1; j<i+2; j++){			
			d[i][j][n].d = p[n-1][i][j-1].d;
			d[i][j][n].e = p[n-1][i][j-1].e;
		}
	}

	for(i=0; i<r; i++){
		for(k=n-1; k>=1; k--){
			for(j=1; j<i+2; j++){			
				d[i][j][k] = bigdouble_mult(bigdouble_mult(b[n][k],p[k-1][i][j-1]), p1[n-k][i][j-1]);
			}
		}
	}

	for(i=0; i<r; i++){
		for(k=n-1; k>=1; k--){
			for(j=1; j<i+2; j++){
				if( d[i][j][k].e >= d[i][j][k+1].e ){				
					y = d[i][j][k].e - d[i][j][k+1].e;
					if( y <= 300 ){				
						d[i][j][k].d += d[i][j][k+1].d / pow(10.0, y);
					}
				} else {
					d[i][j][k].e = d[i][j][k+1].e;
					y = d[i][j][k+1].e - d[i][j][k].e;
					if( y <= 300 ){										
						d[i][j][k].d = d[i][j][k+1].d + d[i][j][k].d / pow(10.0, y);
					}else{
						d[i][j][k].d = d[i][j][k+1].d;
					}
				}

				if( d[i][j][k].d >= 10.0 ){
					d[i][j][k].d /= 10.0;
					d[i][j][k].e ++;
				}

				//d[i][j][k] += d[i][j][k+1];
			}
		}
	}

	for(i=0; i<r; i++){
		for(j=1; j<i+2; j++){			
			for(k=1; k<=n; k++){
				//d[i][j][k] = bigdouble_mult(d[i][j][k], b[l][j]); //noorder
				d[i][j][k] = bigdouble_mult(d[i][j][k], perm[j]); //order
			}
		}
	}

	*e = d;
	return;
}


//calculate all e-values
void comput_order_ALL_EVALUE(int l, int n, int r){
	int i,j;
	bigdouble ** comKKprime;
	bigdouble *** pLKKprime; //pLKKprime[0], p(k, k')^1
	bigdouble *** pLKKprime_1;
	bigdouble *** pKKN;  //pKKprimeNprime

	bigdouble * perm; //permutation l*(l-1)*(l-2)*...*(l-k'+1)

	//permutation l*(l-1)*(l-2)*...*(l-r+1)
	permut(&perm, l, r);

	//calculate 2-d combination -- k, k'
	comb2d(&comKKprime, l);

	//calculate p(l, k, k') and powers
	comput_order_pLKKprime(comKKprime, &pLKKprime, &pLKKprime_1, l, n, r, perm);

	//calculate e values
	comput_order_pKKN(&pKKN, comKKprime, pLKKprime, pLKKprime_1, l, n, r, perm);
	
	//get evalues in simple format
	comput_EVALUES(pKKN, l, n, r);

	if( comKKprime ){
		for(i=0; i<=l; i++)
			if( comKKprime[i] )
				free(comKKprime[i]);
		free(comKKprime);
	}

	if( pLKKprime ){
		for(i=0; i<n; i++){
			for(j=0; j<r; j++){
				if( pLKKprime[i][j] )
					free( pLKKprime[i][j] );
			}
			free( pLKKprime[i] );
		}
		free( pLKKprime );
	}

	if( pLKKprime_1 ){
		for(i=0; i<n; i++){
			for(j=0; j<r; j++){
				if( pLKKprime_1[i][j] )
					free( pLKKprime_1[i][j] );
			}
			free( pLKKprime_1[i] );
		}
		free( pLKKprime_1 );
	}

	return;
}




//zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
//clone r2 to r1
void clone_rnodelist(rnode *r1, rnode *r2, int n_genome){
	int t, p, s;
	rnode *newnode, *curr, *tail;

	if( ! r2->next )
		return;
	
	curr = r2->next;
	while( curr ){
		//clone this node 
		if( !(newnode = malloc(sizeof(rnode))) )
			FPFE1("out of memory for newnode\n");
		newnode->next = NULL;
		//newnode->s    = NULL;

		newnode->l = curr->l;
		newnode->n = curr->n;
		//newnode->avl = curr->avl;
		newnode->eval = curr->eval;

		//determine set
		 if( ( (newnode->set) = malloc( newnode->l *sizeof(int))) ){
			 for(s=0; s<newnode->l; s++)
				(newnode->set)[s] = (curr->set)[s];
		 } else
			 FPFE1("out of memory for newnode->set\n");

		 //determine  chr low high numg  posg
		 //if( !( (newnode->chr) = malloc(n_genome * sizeof(int))) )
		//	 FPFE1("out of memory for newnode->chr\n");
		 if( !( (newnode->low) = malloc(n_genome * sizeof(int))) )
			 FPFE1("out of memory for newnode->low\n");
		 if( !( (newnode->high) = malloc(n_genome * sizeof(int))) )
			 FPFE1("out of memory for newnode->high\n");
		 if( !( (newnode->numg) = malloc(n_genome * sizeof(int))) )
			 FPFE1("out of memory for newnode->numg\n");
		 if( !( (newnode->posg) = malloc(n_genome * sizeof(int *))) )
			 FPFE1("out of memory for newnode->posg\n");

		for(p=0; p<n_genome; p++){
			//(newnode->chr)[p] = (curr->chr)[p];
			(newnode->posg)[p]= (curr->posg)[p];
			(newnode->numg)[p]= (curr->numg)[p];
			(newnode->low)[p] = (curr->low)[p];
			(newnode->high)[p] =(curr->high)[p];

			if((curr->numg)[p] != 0){
				if( !( (newnode->posg)[p] = malloc((curr->numg)[p] * sizeof(int))) )
					FPFE1("out of memory for newnode->posg[]\n");

				for(t=0; t<(curr->numg)[p]; t++){
					(newnode->posg)[p][t] = (curr->posg)[p][t];
				}
			}
		}

		if( ! r1->next ){
			tail = r1->next = newnode;
		} else {
			tail->next = newnode;
			tail = newnode;
		}

		curr = curr->next;
	}

	return;
}

//do all combinations recurrsively
void rcomb_rnode(rnode *rn, int K, int **xchr, int n_genome, int n, int u){
	int p;
	rnode * newnode, *burr, *curr;

	//put this result into a rnode
	if( !(newnode = malloc(sizeof(rnode))) )
		FPFE1("out of memory for newnode\n");
	newnode->next = NULL;

	newnode->l = 1;
	newnode->n = allind;
	newnode->eval = EVALUE[1][allind];

	//determine set
	if( ( (newnode->set) = malloc( sizeof(int))) ){
		(newnode->set)[0] = xchr[n][tmp_K[n]];
	} else
		FPFE1("out of memory for newnode->set\n");

	//determine  chr low high numg  posg
	if( !( (newnode->low) = malloc(n_genome * sizeof(int))) )
		FPFE1("out of memory for newnode->low\n");
	if( !( (newnode->high) = malloc(n_genome * sizeof(int))) )
		FPFE1("out of memory for newnode->high\n");
	if( !( (newnode->numg) = malloc(n_genome * sizeof(int))) )
		FPFE1("out of memory for newnode->numg\n");
	if( !( (newnode->posg) = malloc(n_genome * sizeof(int *))) )
		FPFE1("out of memory for newnode->posg\n");

	for(p=0; p<n_genome; p++){
		(newnode->posg)[p]= NULL;
		(newnode->numg)[p]= 0;

		if( allcomb[u][p] ){
			(newnode->numg)[p]= 1;
			if( !( (newnode->posg)[p] = malloc( sizeof(int))) )
				FPFE1("out of memory for newnode->posg[]\n");
			(newnode->posg)[p][0] = (newnode->high)[p] = (newnode->low)[p]  = tmp_K[p];
		}
	}

	//add the new node to the linked list
	burr = rn;
	curr = rn->next;

	while( curr != NULL && curr->l >= newnode->l ){
		burr = burr->next;
		curr = curr->next;
	}

	if( curr == NULL ){
		burr->next = newnode;
	} else {
		burr->next = newnode;
		newnode->next = curr;
	}
				
	return;
}

//find intersections using a fast (O(nk)) approach
//write results to a linked list and don't output the results directly
void rcomb_intersect_fast(int **xchr, int n_genome, int k, rnode * rn, int xx, int yy){
	int  zeros, m, n, i, j, x,  count; 
	rnode *curr, *burr, *newnode;

/*
int ii,jj;
fprintf(stdout, "\n\nrecur_comb_intersect_fast>>>> ");//////////////////
for(i=0; i<n_genome; i++){
	if( n_sarr[i] == 0 ){
		fprintf(stdout, "\n%2d: (0) ", i+1);
	} else {
		fprintf(stdout, "\n%2d: (%2d) ", i+1, n_sarr[i]);
		for(j=0; j<n_sarr[i]; j++)
			fprintf(stdout, " %1d", sarr[i][j]);
	}
}/////////////////////////
*/


	x = zeros= allindex[0];
	m = n_sarr[zeros];

	for(i=0; i<m; i++){
		cstat[ xchr[x][sarr[x][i]] ] = 1;
	}

	for(i=zeros+1; i<n_genome; i++){
		if( n_sarr[i] == 0 ){
			zeros ++;
		} else {
			j = i - zeros;
			for(n=0; n<n_sarr[i]; n++){
				if( cstat[ xchr[i][sarr[i][n]] ] == j )
					cstat[ xchr[i][sarr[i][n]] ] ++;
			}

			if( cstat[ xchr[xx][yy] ] <= j ){ //subset is already empty
				return; 
			}
		}
	}

	//count number of unique paralogs in current result
	for(count=n=0; n<n_tarr; n++){
		if( cstat[ tarr[n] ] == n_genome-zeros ){
			sarr[n_genome][count] = tarr[n];
			count++;
		} else {
			cstat[ tarr[n] ] = 0;
		}
	}

	if(count == 1)
		return;

	//put this result into a rnode
	if( !(newnode = malloc(sizeof(rnode))) )
		FPFE1("out of memory for newnode\n");
	newnode->next = NULL;

	newnode->l = count;
	newnode->n = allind;
	newnode->eval = EVALUE[count][allind];

	//determine set
	 if( ( (newnode->set) = malloc(count * sizeof(int))) ){
		for(i=0; i<count; i++)
			(newnode->set)[i] = sarr[n_genome][i];
	 } else
		 FPFE1("out of memory for newnode->set\n");

	 //determine  chr low high numg  posg
	 if( !( (newnode->low) = malloc(n_genome * sizeof(int))) )
		 FPFE1("out of memory for newnode->low\n");
	 if( !( (newnode->high) = malloc(n_genome * sizeof(int))) )
		 FPFE1("out of memory for newnode->high\n");
	 if( !( (newnode->numg) = malloc(n_genome * sizeof(int))) )
		 FPFE1("out of memory for newnode->numg\n");
	 if( !( (newnode->posg) = malloc(n_genome * sizeof(int *))) )
		 FPFE1("out of memory for newnode->posg\n");

	for(i=0; i<n_genome; i++){
		(newnode->posg)[i]= NULL;
		(newnode->numg)[i]= 0;

		if(n_sarr[i] ){
				(newnode->numg)[i]= count;
				if( !( (newnode->posg)[i] = malloc(count * sizeof(int))) )
					FPFE1("out of memory for newnode->posg[]\n");
	
				m=0;
				for(j=0; j<n_sarr[i]; j++){
					if( cstat[ xchr[i][sarr[i][j]] ] ){
						(newnode->posg)[i][m] = sarr[i][j];
						m++;
					}
				}
				(newnode->low)[i]  = (newnode->posg)[i][0];
				(newnode->high)[i] = (newnode->posg)[i][m-1];
		}
	}

	 //add the new node to the linked list
	 burr = rn;
	 curr = rn->next;

	 //while( curr != NULL && curr->eval <= newnode->eval ){
	 while( curr && curr->l >= newnode->l ){
		 burr = burr->next;
		 curr = curr->next;
	 }

	 if( curr == NULL ){
		 burr->next = newnode;
	 } else {
		 burr->next = newnode;
		 newnode->next = curr;
	 }	

/*
 	fprintf(stdout, "\n<j= %d k= %d>", yy, k); /////////////////////////////////////////
	curr = newnode;
	while(curr){
		fprintf(stdout, "\nCluster: ");
		for(ii=0; ii<curr->l; ii++)
			fprintf(stdout, " %1d", curr->set[ii]);

		fprintf(stdout, "\n\tExpect = %10.4e, Size = %1d/%1d, Appear = %1d/%1d",
					   curr->eval, curr->l, ave_L, curr->n, n_genome);
		//*
		for(ii=0; ii<n_genome; ii++){
			if( curr->numg[ii] ){
				fprintf(stdout, "\n\t (Chr:%1d S%1d E%1d):",  ChrNO[ii], curr->low[ii], curr->high[ii]);
				for(jj=0; jj<curr->numg[ii]; jj++)
					fprintf(stdout, " %2d (%2d)", xchr[ii][curr->posg[ii][jj]], curr->posg[ii][jj]);
			} else {
				fprintf(stdout, "\n@@");
			}
		}

		curr = curr->next;
	}
	fprintf(stdout, "\nrecur_comb_intersect_fast<<<<\n "); ///////////////////////////////////////////
*/

	return;
}

//compare two ints for qsort
int comp_ints(const int * n1, const int * n2){
	if( *n1 < *n2 ) {
		return -1;
	} else if( *n1 == *n2) {
		return  0;
	} else
		return  1;
}

//binary search: given an id, return the index
int binary_id_search(int * co, int cnt, int id){
	int low, high, curr;

	low=0;
	high=cnt-1;
	while( low <= high ){
		curr= (low+high)/2;

		if( co[curr] == id ){
			return curr;
		} else if ( co[curr] < id ){
			low=curr+1;
		} else {
			high=curr-1;
		}
	}

	return -1;
}


//window sliding
void slide_window(int **xchr, int n_genome, int *n_gene, rnode *r1, rnode *r2, int k, int i, int j, int u, int rr){
	int     K, s, f, g, t, np,  x;//z,
	rnode *rncurr;

	K = k+1;
		
	if( j - rr <= k ){//new gene is not too far away from current sets.
		//find new intersection
		rncurr = r1->next;
		while(rncurr ){
			if( j - rncurr->high[i] <= k ){ //new gene is not too far away from current genes.
				//s = 0;
				//while(s< rncurr->l && rncurr->set[s] != xchr[i][j] )
				//	s++;
				
				//if( s == rncurr->l ) { //there is no paralog.
					memset(n_sarr, 0, sizeof(int)*(n_genome+1));					

					//determine the first set & determine the unique set
					s= rncurr->numg[i]-1;
					while( s>=0 && (j - rncurr->posg[i][s] < K) )
						s--;

					for(f=0; f< rncurr->numg[i] -s -1; f++){
						sarr[i][f] = rncurr->posg[i][f +s +1];
						tarr[f] = xchr[i][sarr[i][f]];
					}
					sarr[i][f] = j;
					tarr[f] = xchr[i][j];

					n_tarr = n_sarr[i] = f+1;

					//sort the unique set
					qsort(tarr, n_tarr, sizeof(int), (void *)comp_ints);

					//determine other chrs	
					for(x=1; x<allind; x++){
						s = allindex[x];
						t = tmp_K[s];

						if( t > rncurr->high[s]+k || t + k < rncurr->low[s] )
							break;

						f=0;
						while(f<rncurr->numg[s] && rncurr->posg[s][f] < t-k )
							f++;
						g=0;
						while(f<rncurr->numg[s] && rncurr->posg[s][f] <= t+k ){
							if( binary_id_search(tarr, n_tarr, xchr[s][ rncurr->posg[s][f] ]) >= 0 ){
								sarr[s][g] = rncurr->posg[s][f];
								g++;
							}	
							f++;
						}						
						
						f=0;
						while(f < g && sarr[s][f] < t ){
							f++;
						}
						
						for(np=g; np>f; np--){
							sarr[s][np] = sarr[s][np-1];
						}
						sarr[s][f] = t;

						n_sarr[s] = g+1;
					}						

					if( x == allind ){						
						memset(cstat, 0, sizeof(int)*(ggmax+1));

						//find the only possible intersection 
						rcomb_intersect_fast(xchr, n_genome, k, r2, i, j);							
					}
				//} 				
			}

			rncurr = rncurr->next;
		}
	}
	
	if( !r2->next )
		rcomb_rnode(r2, K, xchr, n_genome, i, u);
/*
curr = r2->next; //////////////////////////
while(curr){
		fprintf(stdout, "\nnew gene Cluster: ");
		for(ii=0; ii<curr->l; ii++)
			fprintf(stdout, " %1d", curr->set[ii]);

		fprintf(stdout, "\n\tExpect = %10.4e, Size = %1d/%1d, Appear = %1d/%1d",
					   curr->eval, curr->l, ave_L, curr->n, n_genome);
		//*
		for(ii=0; ii<n_genome; ii++){
			if( curr->numg[ii] ){
				fprintf(stdout, "\n\t (Chr:%1d S%1d E%1d):",  ChrNO[ii], curr->low[ii], curr->high[ii]);
				for(jj=0; jj<curr->numg[ii]; jj++)
					fprintf(stdout, " %2d (%2d)", xchr[ii][curr->posg[ii][jj]], curr->posg[ii][jj]);
			} else {
				fprintf(stdout, "\n@@");
			}
		}

		curr = curr->next;
}///////////////////////////////////////////
*/
	return;
}

//Find maximum k that's possible
int find_max_k(double *** t, int l, int np, int kp, double p){
	l--;
	while( l > kp ){
		if( t[l][kp][np] <= p ){
			return l;
		}
		l--;
	}

	return kp;
}

//search for syntenies with zz Method
void syn_finder_zz(int **xchr, int **xpos, char **xname, char **xnomename, int n_genome,
					  int *n_gene, int xmax, int k, char *ofname){
	int  i, j,  z, K,   comb, u,  rr, s, L, t, x; //f,
	rnode *rnhead, *rnhead2, *rnhead3, *result;
	int * cgpos, n_cgpos; 

//	time_t  time1, time2;
	
	if( !(cgpos = malloc((ave_L)*sizeof(int))) )
		FPFE1("out of memory for cgpos\n");

	//allocate space for global tmp_K
	if( !(tmp_K = malloc(ave_L * sizeof(int))) )
			FPFE1("out of memory for tmp_K[]\n");	
	
	if( !(tmpx=malloc(n_genome*sizeof(int))) )
		FPFE1("out of memory for tmpx\n");

	if( !(cstat = malloc( (xmax+1) *sizeof(int))) )
		FPFE1("out of memory for cstat\n");
	
	if( !(allindex = malloc( n_genome * sizeof(int))) )
		FPFE1("out of memory for allindex\n");

    for(comb=1, i=1; i<=n_genome; i++)
		comb *= 2;

	if( !(allcomb=calloc(comb, sizeof(int *))) )
		FPFE1("out of memory for allcomb\n");
	for(i=0; i<comb; i++)
		if( !(allcomb[i] = calloc(n_genome,sizeof(int))) )
			FPFE1("out of memory for allcomb[]\n");

	if( !(rnhead = malloc(sizeof(rnode))) )
		FPFE1("out of memory for head\n");
	if( !(rnhead2 = malloc(sizeof(rnode))) )
		FPFE1("out of memory for head2\n");
	if( !(rnhead3 = malloc(sizeof(rnode))) )
		FPFE1("out of memory for head3\n");
	if( !(result = malloc(sizeof(rnode))) )
		FPFE1("out of memory for result\n");
	
	//allocate space for global tarr
	if( !(tarr = malloc(ave_L * sizeof(int))) )
			FPFE1("out of memory for tarr[]\n");	
	
	//allocate space for global sarr
	if( !(sarr = malloc( (n_genome+1) * sizeof(int *))) )
		FPFE1("out of memory for sarr\n");
	for(i=0; i<=n_genome; i++){
		if( !(sarr[i] = malloc(ave_L * sizeof(int))) )
			FPFE1("out of memory for sarr[]\n");
	}
	if( !(n_sarr = malloc( (n_genome+1) * sizeof(int))) )
		FPFE1("out of memory for n_sarr\n");

	allind = 0;
	recur_comb(n_genome, 0);

	result->next = NULL;

	//main loop
	for(u=0; u<comb; u++){
	//for(u=10; u<11; u++){
		//time1 = time(NULL);
		//fprintf(stdout, "\nallcomb: ");
		for(allind=0, j=0; j<n_genome; j++){
			//fprintf(stdout, " %2d", allcomb[u][j]);
			if( allcomb[u][j] == 1){
				allindex[ allind ] = j;
				allind ++;
			}
		}

		//if(allind < MIN_N)
		if(allind < 2) continue;

		x=allindex[0];
		//for(n_cgpos=i=0; i<xmax; i++){
		for(n_cgpos=0, i=1; i<=n_gene[x]; i++){
			z = xchr[x][i]; 
			if( z != -1 ){
				for(j=1; j<allind; j++)
					if( xpos[z][ allindex[j] ] == -1 )
						break;

				if( j == allind ){
					cgpos[n_cgpos] = i;					
					n_cgpos++;
				}
			}
		}

		//fprintf(stdout, "\nallind=%d n_cgpos=%d",allind, n_cgpos);

		if( n_cgpos <= 1 )
			continue;
		
		i= allindex[0];

		L = (ave_L<=n_gene[i])?ave_L:n_gene[i];

		if( dist_k < 0 ){
			fprintf(stderr, "\ndistance Cutoff is negative ?!\n"), fflush(NULL), exit(0);
		} else if (dist_k == 0 || dist_k >= ave_L){
			//Find maximum k that's possible
			s = find_max_k(allEVALUES, ave_L, allind, n_cgpos, EVALUE_CUTOFF)+1;

			if( s < L )
				L = s;
		} else {
			if( dist_k  < L )
				L = dist_k;
		}

		//fprintf(stdout, "\nl=%d np=%d kp=%d s=%d L=%d n_gene=%d", ave_L, allind, n_cgpos, s, L, n_gene[i]);
		//L = 101;
		
		k=0;
		K=k+1;

		EVALUE = allEVALUES[k];

		//slide window
		rnhead->next = NULL;
		rnhead2->next = NULL;
		rnhead3->next = NULL;		

		//collect all orthologs and paralogs
		s = j = cgpos[0];
		tmp_K[i]= j;
		z = xchr[i][j];
		for(t=1; t<allind; t++){
			tmp_K[allindex[t]] = xpos[z][allindex[t]];					
		}		
		rcomb_rnode(rnhead3, K, xchr, n_genome, i, u);			
	
		//extend window
		for(k=1; k<L && s+k<=n_gene[i]; k++){
			EVALUE = allEVALUES[k];			

			clone_rnodelist(rnhead, rnhead3, n_genome);	
			
			rr = s;
			for(x=1; x<n_cgpos; x++){
				//collect all orthologs and paralogs
				j = cgpos[x];					
				tmp_K[i] = j;
				for(t=1; t<allind; t++){						
					tmp_K[allindex[t]] = xpos[xchr[i][j]][allindex[t]];							
				}				
				
				slide_window(xchr, n_genome, n_gene, rnhead, rnhead2, k, i, j, u, rr);
				combine2lists(rnhead, rnhead2, n_genome);
				remove_repeat(rnhead, n_genome);
				rr = j;				
			}
			
			combine2lists_evalcut(result, rnhead, n_genome);	
		}
		//time2 = time(NULL);
		//fprintf(stdout, "\n\t %7.2f mins", (double)difftime(time2, time1)/60), fflush(NULL);	
	}////start search from each genome once

	//remove repeats from a linked list of rnodes
	remove_repeat(result, n_genome);

	output_rnodes(xchr, xnomename, n_genome, ofname, result);

	return;
}

//ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER ORDER 
typedef struct ordersyn {
	int n; //number of elements
	int m; //number of chrosomes containing the syn
	double p; //probability
	int * c; //record which chromosomes contain the syn, 1 or 0
	int * s; //cluster ID's in current syn
	struct ordersyn * prev;
	struct ordersyn * next;
} ordersyn;

ordersyn * oresult, *otail;
double omin;


int call_eppstein(int **dag, int i, int k, int allind, int *allindex, int **xchr, int **xpos, 
				   char **xname, char **xnomename, int n_genome,int *n_gene, char *ofname)
{
	FILE * gfile, *inf;
	int j, m, n, p, q, r;
	double score;

	char *ptr, *ptr1;
	char cmdline[300], buffer[4096];
	ordersyn *curr, *burr;
	
	//output graph file in correct format
	if( !(gfile = fopen("tmpgraph", "w")) )
		fprintf(stdout, "tmpgraph cannot be opened for writing!\n"), fflush(NULL), exit(0);
	fprintf(gfile, "n %d", k+2);

	/*
	if( i>0 ){
		for(p=0; p<=n_gene[allindex[0]]; p++){
			fprintf(stdout, "\n");
			for(q=0; q<=n_gene[allindex[0]]; q++)
				fprintf(stdout, "%d ", dag[p][q]);
		}
	}
	fprintf(stdout, "\n");
	fflush(NULL);
	*/

	//count the edges
	m = n = 0;
	for(p=i; p<=n_gene[allindex[0]] && p <= i+k; p++){
		if( p>i && dag[i][p] == 0 )
			continue;

		if ( p>i )
			n++;

		for(q=p+1; q<=n_gene[allindex[0]] && q<= i+k; q++){
		//for(q=p+1; q<=n_gene[allindex[0]]; q++){
			if(dag[p][q] == 1 && dag[i][q] == 1 )
				m++;
		}
	}

	//fprintf(gfile, "\nm %d", m+k+1);
	fprintf(gfile, "\nm %d", m+n+1);
	fprintf(gfile, "\ns 1");
	fprintf(gfile, "\nt %d", k+2);

	//output edges
	//fprintf(gfile, "\na %d %d 0", 1, k+2);   //edges to sink
	for(p=i; p<=n_gene[allindex[0]] && p <= i+k; p++){
		if( p>i && dag[i][p] == 0 )
			continue;

		fprintf(gfile, "\na %d %d 0", p-i+1, k+2);   //edges to sink
		for(q=p+1; q<=n_gene[allindex[0]] && q<= i+k; q++){
		//for(q=p+1; q<=n_gene[allindex[0]]; q++){
			if(dag[p][q] == 1 && dag[i][q] == 1 ){
				fprintf(gfile, "\na %d %d -1", p-i+1, q-i+1);				
			}
		}
	}

	fprintf(gfile, "\n");
	fclose(gfile);

	sprintf(cmdline, "EPPSTEIN tmpgraph %d -paths", NUM_OUTPUT);

	//call Eppstein's algorithm
	system(cmdline); 
	
	//read in eppstein results
	if( !(inf = fopen("tmpeppout", "r")) ){
		//fprintf(stdout, "File tmpeppout cannot be opened for reading!\n"), fflush(NULL), exit(0);
		return 0;
	}

	while( fgets(buffer, 4096, inf) ){
		if( strstr(buffer, "N=") ){
			if( ptr = strstr(buffer, "(Cost:") ){
				if( sscanf(ptr, "(Cost: %d) ", &r) != 1 )
					fprintf(stdout, "\nCan't correctly read cost in line:\n%s\n\n", buffer), fflush(NULL), exit(0);

				score = EVALUE[-r +1][allind];

				if( score >= omin ){ //impossible to be top
					break;
				} else {
					//read in the result and save in the last node
					curr = otail;

					otail = otail->prev;
					otail->next =NULL;

					curr->prev = NULL;

					curr->n = -r+1;
					curr->m = allind;
					curr->p = score;

					//curr->c
					if( !(curr->c = calloc(n_genome, sizeof(int))) )
						fprintf(stdout, "\nout of memory\n\n"), fflush(NULL), exit(0);
					for(p=0; p<n_genome; p++){
						(curr->c)[p] = 0;
					}
					for(p=0; p<allind; p++)
						(curr->c)[allindex[p]] = 1;

					//curr->s, read in the path
					if( !(curr->s = calloc(curr->n, sizeof(int))) )
						fprintf(stdout, "\nout of memory\n\n"), fflush(NULL), exit(0);

					p = allindex[0];

					(curr->s)[0] = xchr[p][i];

					while( (ptr - buffer) >= 0 && *ptr != '-' ) ptr --;
					ptr --;
					while( (ptr - buffer) >= 0 && *ptr != '-' ) ptr --;
					*(ptr+1) = '\0';

					ptr1 = buffer;
					while( *ptr1 != '-' )
						ptr1++;
					ptr1++;

					if( ptr1 > ptr ){ //only one node
						//fprintf(stderr, "\n\n%s\n", buffer);
						//fprintf(stderr, "Something wrong here?\n\n"), fflush(NULL), exit(0);
					} else {
						j=1;
						while( *ptr1 != '\0' ){
							ptr = ptr1;
							while( *ptr != '-' ) ptr++;
							*ptr = '\0';
							(curr->s)[j] = xchr[p][ i + atoi( ptr1 ) - 1];
							j++;
							ptr1 = ptr + 1;
						}	
					}

					//insert the node into the list
					if( oresult->p > score ){
						oresult->prev = curr;
						curr->next = oresult;

						oresult = curr;						
					} else {
						burr = oresult;

						while( burr && burr->p <= score ){
							burr = burr->next;
						}

						if( burr ){
							burr->prev->next = curr;
							curr->prev = burr->prev;
							curr->next = burr;
							burr->prev = curr;
						} else {
							otail->next = curr;
							curr->prev = otail;

							otail = curr;
						}
					}

					omin = otail->p;
				}
			} else {
				fprintf(stdout, "\nNo cost in this line:\n%s\n\n", buffer), fflush(NULL),exit(0);
			}
		}		
	}
	fclose(inf);	

	//remove("tmpeppout");

	return 0;
}

//output results
void output_order_results(int **xchr,int **xpos, char **xnomename, int n_genome, char *ofname, ordersyn * r)
{
	FILE * outf;
	int i, j, p, q, n;

	ordersyn * curr;

	if( ! r->next )
		return;

	if( !(outf = fopen(ofname, "a")) )
		FPFE2("File %s can't be opened for appending\n", ofname);

	curr = r;

	n=1;
	while(curr){
		if( curr->n < 2 )
			break;

		fprintf(outf, "Cluster: ");
		for(i=0; i<curr->n; i++)
			fprintf(outf, " %1d", curr->s[i]);

		fprintf(outf, "\n\tExpect = %10.4e, Size = %1d/%1d, Appear = %1d/%1d",
						   curr->p, curr->n, ave_L, curr->m, n_genome);
		//*
		p = curr->s[0];
		q = curr->s[curr->n -1];
		for(i=0; i<n_genome; i++){
			if( curr->c[i] ){
				fprintf(outf, "\n\t%15s (Chr:%1d S%1d E%1d):", xnomename[i], ChrNO[i], xpos[p][i], xpos[q][i]);
				for(j=0; j<curr->n; j++)
					fprintf(outf, " %2d (%2d)", curr->s[j], xpos[ curr->s[j] ][i]);
			} else {
				fprintf(outf, "\n@@");
			}
		}

		curr = curr->next;
		n++;

		fprintf(outf, "\n\n");
	}

	fclose(outf);

	return;

}


//search for syntenies with genes in same order
void syn_finder_order(int **xchr, int **xpos, char **xname, char **xnomename, int n_genome,
					  int *n_gene, int xmax, int k, char *ofname)
{
	int  i, j, comb, u, x; 

	int maxng; //the max gene position on any chr
	int **dag;
	int a, b, c, d;

	ordersyn * curr;

	if( !(tmpx=malloc(n_genome*sizeof(int))) )
		FPFE1("out of memory for tmpx\n");

	if( !(allindex = malloc( n_genome * sizeof(int))) )
		FPFE1("out of memory for allindex\n");
    for(comb=1, i=1; i<=n_genome; i++)
		comb *= 2;
	if( !(allcomb=calloc(comb, sizeof(int *))) )
		FPFE1("out of memory for allcomb\n");
	for(i=0; i<comb; i++)
		if( !(allcomb[i] = calloc(n_genome,sizeof(int))) )
			FPFE1("out of memory for allcomb[]\n");

	allind = 0;
	recur_comb(n_genome, 0);

	//determine maxng;
	for(maxng=-1, i=0; i<n_genome; i++){
		if( n_gene[i] > maxng )
			maxng = n_gene[i];
	}
	if( maxng <= -1 ) //no genes at all?
		return;  

	//allocate space for dag
	if( !(dag = calloc(maxng+1, sizeof(int *))) )
		FPFE1("out of memory for dag\n");
	for(i=0; i<=maxng; i++){
		if( !(dag[i] = calloc(maxng+1, sizeof(int))) )
			FPFE1("out of memory for dag[]\n");
	}

	//initialize oresult, otail, omin
	oresult = otail = NULL;
	omin    = 1e200;

	for(j=0; j<NUM_OUTPUT; j++){
		if( !oresult ){
			if( !(oresult = otail = calloc(1, sizeof(ordersyn))) )
				fprintf(stdout, "out of memory for oresult\n"), fflush(NULL),exit(0);

			otail->p = 1e200;
			otail->n = -1; 
			otail->m = -1; 
			otail->prev = NULL;
			otail->next = NULL;
			otail->c = NULL;
			otail->s = NULL;
			if( !(otail->c = calloc(n_genome, sizeof(int))) )
				fprintf(stdout, "out of memory for otail->c\n"), fflush(NULL),exit(0);
			if( !(otail->s = calloc(k+1, sizeof(int))) )
				fprintf(stdout, "out of memory for otail->s\n"), fflush(NULL),exit(0);				
		} else {
			if( !(curr = calloc(1, sizeof(ordersyn))) )
				fprintf(stdout, "out of memory for curr\n"), fflush(NULL),exit(0);

			curr->p = 1e200;
			curr->n = -1; 
			curr->m = -1; 
			curr->prev = NULL;
			curr->next = NULL;
			curr->c = NULL;
			curr->s = NULL;
			if( !(curr->c = calloc(n_genome, sizeof(int))) )
				fprintf(stdout, "out of memory for curr->c\n"), fflush(NULL),exit(0);
			if( !(curr->s = calloc(k+1, sizeof(int))) )
				fprintf(stdout, "out of memory for curr->s\n"), fflush(NULL),exit(0);

			curr->prev = otail;
			otail->next = curr;

			otail = curr;
		}
	}

	//main loop
	for(u=0; u<comb; u++){
	//for(u=10; u<11; u++){
		//time1 = time(NULL);
		//fprintf(stdout, "\nallcomb: ");
		for(allind=0, j=0; j<n_genome; j++){
			//fprintf(stdout, " %2d", allcomb[u][j]);
			if( allcomb[u][j] == 1){
				allindex[ allind ] = j;
				allind ++;
			}
		}

		//if(allind < MIN_N)
		if(allind < 2) continue;

		//initialize dag[][]
		for(i=0; i<=maxng; i++)
			for(j=0; j<=maxng; j++)
				dag[i][j] = 0;

		//construct dag[][]
		x=allindex[0];
		for(i=0; i<=n_gene[x]; i++){
			a = xchr[x][i];
			if( a < 0 )
				continue;

			for(j=i+1; j<=n_gene[x] && j<=i+k; j++){				
			//for(j=i+1; j<=n_gene[x]; j++){				
				b = xchr[x][j];
				if( b < 0 )
					continue;

				for(c=1; c<allind; c++){
					if( xpos[b][ allindex[c] ] < 0 || xpos[a][ allindex[c] ] < 0 )
						break;

					d = xpos[b][ allindex[c] ] - xpos[a][ allindex[c] ];
					if( !( d > 0 && d <=k) )
						break;					
				}

				if( c == allind ){
					dag[i][j] = 1;
				}
			}
		}
		
		EVALUE = allEVALUES[k];

		//call eppsteins k by k
		for(i=0; i<=n_gene[x]; i++){
		//for(i=4; i<=4; i++){
			//fprintf(stdout, "i=%d \n", i), fflush(NULL);
			if( xchr[x][i] < 0 )
				continue;

			call_eppstein(dag, i, k, allind, allindex, xchr, xpos, xname, xnomename, n_genome,n_gene, ofname);

			/*
			curr = oresult;
			while( curr->n > 0 ){
				fprintf(stdout, "\n");
				for(c=0; c<curr->n; c++)
					fprintf(stdout, " %d", curr->s[c]);

				curr = curr->next;
			}
			//*/
		}		
	}////start search from each genome once

	output_order_results(xchr, xpos, xnomename, n_genome, ofname, oresult);

	return;					  
}


//do all combinations recurrsively
void recur_comb_3(int nGenomes, int n){
	int i, j;

	for(i=1; i>=-1; i--){
		tmpx[n] = i;

		if( n == nGenomes-1 ){
			for(j=0; j<nGenomes; j++)
				allcomb[allind][j] = tmpx[j];

			allind ++;
		} else {
			recur_comb_3(nGenomes, n+1);
		}
	}
}

int * tmpepppath; //path from epptmpout

//
int use_eppstein(int **dag, int x, int ntable, int **table, int **xchr, char **xnomename, int n_genome, char *ofname)
{
	FILE * gfile, *inf, *outf;
	int j, m, n, p, q, r, b, d;
	double score;

	char *ptr, *ptr1;
	char cmdline[300], buffer[4096];
	//ordersyn *curr, *burr;
	
	//output graph file in correct format
	if( !(gfile = fopen("tmpgraph", "w")) )
		fprintf(stdout, "tmpgraph cannot be opened for writing!\n"), fflush(NULL), exit(0);
	fprintf(gfile, "n %d", ntable +2);

	//count the edges
	m = n = 0;
	for(p=0; p<ntable; p++){
		for(q=0; q<ntable; q++){		
			if( dag[p][q] == 1 )
				m++;
		}
	}

	//fprintf(gfile, "\nm %d", m+k+1);
	fprintf(gfile, "\nm %d", m + ntable );
	fprintf(gfile, "\ns 1");
	fprintf(gfile, "\nt %d", ntable+2);

	//output edges
	//source ->nodes
	fprintf(gfile, "\na 1 %d 0", 2);

	//nodes -> sink
	for(p=1; p<ntable; p++){
		fprintf(gfile, "\na %d %d 0", p+2, ntable+2);
	}
	//other edges
	for(p=0; p<ntable; p++){
		for(q=0; q<ntable; q++){		
			if( dag[p][q] == 1 )
				fprintf(gfile, "\na %d %d -1", p+2, q+2);				
		}
	}

	fprintf(gfile, "\n");
	fclose(gfile);

	remove("tmpeppout");
	sprintf(cmdline, "EPPSTEIN tmpgraph %d -paths >tmpeppout ", NUM_OUTPUT);

	//call Eppstein's algorithm
	system(cmdline); 
	fflush(NULL);
	
	//read in eppstein results
	if( !(inf = fopen("tmpeppout", "r")) ){
		//fprintf(stdout, "File tmpeppout cannot be opened for reading!\n"), fflush(NULL), exit(0);
		return 0;
	}

	if( !(outf = fopen(ofname, "a")) )
		fprintf(stdout, "File %s can't be opened for appending!\n", ofname), fflush(NULL), exit(0);

	while( fgets(buffer, 4096, inf) ){
		if( strstr(buffer, "N=") ){
			if( ptr = strstr(buffer, "(Cost:") ){
				if( sscanf(ptr, "(Cost: %d) ", &r) != 1 )
					fprintf(stdout, "\nCan't correctly read cost in line:\n%s\n\n", buffer), fflush(NULL), exit(0);

				if( -r+1 < 2 ) // only up to 1 
					break;

					
				while( (ptr - buffer) >= 0 && *ptr != '-' ) ptr --;
				ptr --;
				while( (ptr - buffer) >= 0 && *ptr != '-' ) ptr --;
				*(ptr+1) = '\0';

				ptr1 = buffer;
				while( *ptr1 != '-' )
					ptr1++;
				ptr1++;

				if( ptr1 > ptr ){ //only one node
					//fprintf(stderr, "\n\n%s\n", buffer);
					//fprintf(stderr, "Something wrong here?\n\n"), fflush(NULL), exit(0);
				} else {
					j=0;
					while( *ptr1 != '\0' ){
						ptr = ptr1;
						while( *ptr != '-' ) ptr++;
						*ptr = '\0';
						tmpepppath[j] = atoi( ptr1 ) - 2;
						j++;
						ptr1 = ptr + 1;
					}	
				}

				m=n=0;
				for(p=0; p<n_genome; p++){
					if( allcomb[x][p] ){
						n++;

						/*
						b = 999999;
						d = -1;
						for(q=0; q<j; q++){
							if( table[p][ tmpepppath[q] ] < b ){
								b = table[p][ tmpepppath[q] ];
							} else if ( table[p][ tmpepppath[q] ] > d ){
								d = table[p][ tmpepppath[q] ];
							}
						}

						if( d-b > m )
							m = d-b;
						*/
						b = abs(table[p][ tmpepppath[0] ] - table[p][ tmpepppath[j-1] ] );
						if( b > m )
							m = b;
					}					
				}

				//test output
				//fprintf(stdout, "\nj=%4d \n-r+1=%d ntable=%d\n", j, -r+1, ntable);
				//fprintf(stdout, "\nTmpTable\n");
				//for(p=0; p<n_genome; p++){
				//	for(q=0; q<ntable; q++){
				//		fprintf(stdout, " %4d", table[p][q]);
				//	}
				//	fprintf(stdout, "\n");					
				//}
				//fprintf(stdout, "\nTmpEppPath\n");
				//for(p=0; p<j; p++){
				//	fprintf(stdout, " %4d", tmpepppath[p]);
				//}
				fflush(NULL);

				if( m == 0 )
					fprintf(stdout, "\nm=0\n"), fflush(NULL), exit(0);


				score = allEVALUES[m][j][n];
				if( score > EVALUE_CUTOFF )
					break;

				/////////////////////////
				fprintf(outf, "Cluster: ");
				for(p=0; p<n_genome; p++){
					if( allcomb[x][p] ){						
						for(q=0; q<j; q++)
							fprintf(outf, " %1d", xchr[p][table[p][ tmpepppath[q] ]]);	

						break;
					}					
				}				

				fprintf(outf, "\n\tExpect = %10.4e, Size = %1d/%1d, Appear = %1d/%1d",
								score, j, ave_L, n, n_genome);
				//*	
				//fprintf(outf, "Cluster: ");
				for(p=0; p<n_genome; p++){
					if( allcomb[x][p] ){		
						if( table[p][ tmpepppath[0] ] > table[p][ tmpepppath[j-1] ] ){
							d = table[p][ tmpepppath[0] ];
							b = table[p][ tmpepppath[j-1] ];
						} else {
							b = table[p][ tmpepppath[0] ];
							d = table[p][ tmpepppath[j-1] ];
						}
						fprintf(outf, "\n\t%15s (Chr:%1d S%1d E%1d):", xnomename[p], ChrNO[p], b, d);
						for(q=0; q<j; q++){
							fprintf(outf, " %2d (%2d)", xchr[p][table[p][ tmpepppath[q] ]], table[p][ tmpepppath[q] ]);
						}					
					} else 		
						fprintf(outf, "\n@@");
				}			
				
				fprintf(outf, "\n\n");
				////////////////////////////				
			} else {
				fprintf(stdout, "\nNo cost in this line:\n%s\n\n", buffer), fflush(NULL),exit(0);
			}
		}		
	}
	fclose(inf);	
	fclose(outf);

	//remove("tmpeppout");

	return 0;
}

//>>>>>>>>>>>>********  Top n non-overlapped paths  *****************
int bellman_ford(int g3order, EDGE_t *g3edges, int g3size, int s, int *d, int *pi){
	int i, j;

	//initialize
	for(i=0;i<g3order;i++){
		d[i] =SHRT_MAX;
		pi[i]=-1;
	}
	d[s]=0;

	//calculations
	for(i=1; i<=g3order-1;i++){
		for(j=0;j<g3size;j++){
			if( g3edges[j].w < SHRT_MAX && d[g3edges[j].v2] > d[g3edges[j].v1] + g3edges[j].w ){
				d[g3edges[j].v2] = d[g3edges[j].v1] + g3edges[j].w;
				pi[g3edges[j].v2] = g3edges[j].v1;
			}
		}
	}

	for(j=0;j<g3size;j++){
		if( g3edges[j].w < SHRT_MAX && d[g3edges[j].v2] > d[g3edges[j].v1] + g3edges[j].w )
			return 0;
	}

	return 1;
}

//hold the shortest path recovered
int * spath;
int spath_count;

//print out a bellman_ford path
void bellman_ford_print_path(int *pi, int s, int v){
	if( v == s ){
		spath[spath_count++]=s;
	} else if( pi[v] == -1) {
		spath[spath_count]=-1;
	} else {
		bellman_ford_print_path(pi, s, pi[v]);
		spath[spath_count++]=v;
	}
}


//Output top n non-overlapped paths 
int non_overlap_n(int **dag, int x, int ntable, int **table, int **xchr, char **xnomename, int n_genome, char *ofname, 
				  EDGE_t *tedges, int n_tedges, int *di, int *pi)
{
	FILE  *outf;
	int j, m, n, p, q, b, d;
	double score;

	int tv, te;
	int cond;
	

	//char *ptr, *ptr1;
	//char cmdline[300], buffer[4096];
	//ordersyn *curr, *burr;
	
	//output graph file in correct format

	//total # of nodes
	tv = ntable +2;	

	//count the edges
	te = 0;
	//source to first node
	tedges[te].v1 = 0;
	tedges[te].v2 = 1;
	tedges[te].w  = 0;
	te ++;

	//nodes -> sink
	for(p=1; p<ntable; p++){
		if( te == n_tedges )
			fprintf(stdout, "\n\nn_tedges is set too small\n\n"), fflush(NULL), exit(0);

		tedges[te].v1 = p +1;
		tedges[te].v2 = ntable +1;
		tedges[te].w  = 0;
		te ++;
	}

	//other edges
	for(p=0; p<ntable; p++){
		for(q=0; q<ntable; q++){		
			if( dag[p][q] == 1 ){
				if( te == n_tedges )
					fprintf(stdout, "\n\nn_tedges is set too small\n\n"), fflush(NULL), exit(0);

				tedges[te].v1 = p +1;
				tedges[te].v2 = q +1;
				tedges[te].w  = -1;
				te ++;
			}
		}
	}

	if( !(outf = fopen(ofname, "a")) )
		fprintf(stdout, "File %s can't be opened for appending!\n", ofname), fflush(NULL), exit(0);

	cond = 0;
	while ( cond < NUM_OUTPUT ) {
		//calculate the shortest path between source and sink
		if( bellman_ford(tv, tedges, te, 0, di, pi) == 0 )
			fprintf(stdout, "\nThere are negative cycles in the input graph\n\n"), fflush(NULL), exit(0);

		//print out a bellman_ford path
		spath_count = 0;
		bellman_ford_print_path(pi, 0, ntable +1);

		if( spath_count <= 4 ) //nontrivial path only
			break;

		//put current shortest path into an array
		for(j=1; j<=spath_count -2; j++){
			tmpepppath[j -1] = spath[j] -1;  //????
		}	

		j = spath_count -2;

		m=n=0;
		for(p=0; p<n_genome; p++){
			if( allcomb[x][p] ){
				n++;
				
				b = abs(table[p][ tmpepppath[0] ] - table[p][ tmpepppath[spath_count-3] ] );
				if( b > m )
					m = b;
			}					
		}

		//test output
		//fprintf(stdout, "\nj=%4d \n-r+1=%d ntable=%d\n", j, -r+1, ntable);
		//fprintf(stdout, "\nTmpTable\n");
		//for(p=0; p<n_genome; p++){
		//	for(q=0; q<ntable; q++){
		//		fprintf(stdout, " %4d", table[p][q]);
		//	}
		//	fprintf(stdout, "\n");					
		//}
		//fprintf(stdout, "\nTmpEppPath\n");
		//for(p=0; p<j; p++){
		//	fprintf(stdout, " %4d", tmpepppath[p]);
		//}
		fflush(NULL);

		if( m == 0 )
			fprintf(stdout, "\nm=0\n"), fflush(NULL), exit(0);

		if( m >= ave_L )
			continue;

		score = allEVALUES[m][j][n];
		if( score > EVALUE_CUTOFF )
			break;

		/////////////////////////
		fprintf(outf, "Cluster: ");
		for(p=0; p<n_genome; p++){
			if( allcomb[x][p] ){						
				for(q=0; q<j; q++)
					fprintf(outf, " %1d", xchr[p][table[p][ tmpepppath[q] ]]);	

				break;
			}					
		}				

		fprintf(outf, "\n\tExpect = %10.4e, Size = %1d/%1d, Appear = %1d/%1d",
						score, j, ave_L, n, n_genome);

		for(p=0; p<n_genome; p++){
			if( allcomb[x][p] ){		
				if( table[p][ tmpepppath[0] ] > table[p][ tmpepppath[j-1] ] ){
					d = table[p][ tmpepppath[0] ];
					b = table[p][ tmpepppath[j-1] ];
				} else {
					b = table[p][ tmpepppath[0] ];
					d = table[p][ tmpepppath[j-1] ];
				}
				fprintf(outf, "\n\t%15s (Chr:%1d S%1d E%1d):", xnomename[p], ChrNO[p], b, d);
				for(q=0; q<j; q++){
					fprintf(outf, " %2d (%2d)", xchr[p][table[p][ tmpepppath[q] ]], table[p][ tmpepppath[q] ]);
				}					
			} else 		
				fprintf(outf, "\n@@");
		}			
		
		fprintf(outf, "\n\n");

		cond ++;

		//mask nodes by removing all edges coincident to the nodes
		for(q=0; q<j; q++){
			for(p=0; p<te; p++){
				if( (tedges[p].v1 == tmpepppath[q] +1) || (tedges[p].v2 == tmpepppath[q] +1) )
					tedges[p].w = SHRT_MAX;
			}			
		}	
		
	}

	fclose(outf);

	return 0;
}

//<<<<<<<<<<<<********  Top n non-overlapped paths  *****************


//search for syntenies with genes in same order -- in two directions
void syn_finder_order_2ways(int **xchr, int **xpos, char **xname, char **xnomename, int n_genome,
					  int *n_gene, int xmax, int k, char *ofname)
{
	int  i, j, comb, x, y; 

	int maxng; //the max gene position on any chr
	int **dag;
	int a, b, c, d;
	
	int ** tmptable; //keep all combinations
	int n_tmptable;  //number of combinations

	ordersyn * curr;

	EDGE_t * tmpedges;
	int    n_tmpedges;
	int *  dist, *pi;

	if( !(tmptable = calloc(n_genome, sizeof(int *))) )
		FPFE1("out of memory for tmptable\n");
	for( i=0; i<n_genome; i++ ){
		if( !(tmptable[i] = calloc(k+1, sizeof(int *))) )
			FPFE1("out of memory for tmptable[]\n");
	}

	if( !(tmpx=malloc(n_genome*sizeof(int))) )
		FPFE1("out of memory for tmpx\n");

	if( !(allindex = malloc( n_genome * sizeof(int))) )
		FPFE1("out of memory for allindex\n");
    for(comb=i=1; i<=n_genome; i++)
		comb *= 3;
	if( !(allcomb=calloc(comb, sizeof(int *))) )
		FPFE1("out of memory for allcomb\n");
	for(i=0; i<comb; i++){
		if( !(allcomb[i] = calloc(n_genome+1,sizeof(int))) )
			FPFE1("out of memory for allcomb[]\n");

		allcomb[i][n_genome] = 1;
	}

	allind = 0;
	recur_comb_3(n_genome, 0);
	comb = allind;

	//remove unnecessary combinations
	for(i=1; i<allind; i++){
		for(j=0; j<i; j++){
			for(d=0; d<n_genome; d++){
				if( allcomb[i][d] != - allcomb[j][d] )
					break;
			}

			if( d == n_genome ){
				allcomb[i][n_genome] = 0;
			}
		}

		b=0;
		for(d=0; d<n_genome; d++){
			if( allcomb[i][d] )
				b++;
		}
		if( b < 2 || b < NUM_GENOME_CUTOFF )
			allcomb[i][n_genome] = 0;
	}	

	//determine maxng;
	for(maxng=-1, i=0; i<n_genome; i++){
		if( n_gene[i] > maxng )
			maxng = n_gene[i];
	}
	if( maxng <= -1 ) //no genes at all?
		return;  

	//allocate space for dag
	if( !(dag = calloc(maxng+1, sizeof(int *))) )
		FPFE1("out of memory for dag\n");
	for(i=0; i<=maxng; i++){
		if( !(dag[i] = calloc(maxng+1, sizeof(int))) )
			FPFE1("out of memory for dag[]\n");
	}

	//allocate space for tmpepppath
	if( !(tmpepppath = calloc(k+10, sizeof(int))) )
		FPFE1("out of memory for tmpepppath\n");

	n_tmpedges = maxng * maxng;
	if( !(tmpedges = calloc(n_tmpedges, sizeof(EDGE_t))) )
		FPFE1("out of memory for tmpedges\n");
	if( !(dist = calloc(k+10, sizeof(int))) )
		FPFE1("out of memory for dist\n");
	if( !(pi = calloc(k+10, sizeof(int))) )
		FPFE1("out of memory for pi\n");
	if( !(spath = calloc(k+10, sizeof(int))) )
		FPFE1("out of memory for spath\n");


	//initialize oresult, otail, omin
	oresult = otail = NULL;
	omin    = 1e200;

	for(j=0; j<NUM_OUTPUT; j++){
		if( !oresult ){
			if( !(oresult = otail = calloc(1, sizeof(ordersyn))) )
				fprintf(stdout, "out of memory for oresult\n"), fflush(NULL),exit(0);

			otail->p = 1e200;
			otail->n = -1; 
			otail->m = -1; 
			otail->prev = NULL;
			otail->next = NULL;
			otail->c = NULL;
			otail->s = NULL;
			if( !(otail->c = calloc(n_genome, sizeof(int))) )
				fprintf(stdout, "out of memory for otail->c\n"), fflush(NULL),exit(0);
			if( !(otail->s = calloc(k+1, sizeof(int))) )
				fprintf(stdout, "out of memory for otail->s\n"), fflush(NULL),exit(0);				
		} else {
			if( !(curr = calloc(1, sizeof(ordersyn))) )
				fprintf(stdout, "out of memory for curr\n"), fflush(NULL),exit(0);

			curr->p = 1e200;
			curr->n = -1; 
			curr->m = -1; 
			curr->prev = NULL;
			curr->next = NULL;
			curr->c = NULL;
			curr->s = NULL;
			if( !(curr->c = calloc(n_genome, sizeof(int))) )
				fprintf(stdout, "out of memory for curr->c\n"), fflush(NULL),exit(0);
			if( !(curr->s = calloc(k+1, sizeof(int))) )
				fprintf(stdout, "out of memory for curr->s\n"), fflush(NULL),exit(0);

			curr->prev = otail;
			otail->next = curr;

			otail = curr;
		}
	}

	//k by k scan
	for(i=0; i<=n_gene[0]; i++){
		if( xchr[0][i] < 0 )
			continue;

		n_tmptable = 0;
		for(j=i; j<=n_gene[0] && j<=i+k; j++){
			if( xchr[0][j] < 0 )
				continue;

			tmptable[0][n_tmptable] = j;
			b = xchr[0][j]; 

			for(a=1; a<n_genome; a++){
				tmptable[a][n_tmptable] = xpos[b][a];
			}

			n_tmptable++;
		}

		/*
		fprintf(stdout, "\n\n");
		for(a=0; a<n_genome; a++){
			for(j=0; j<n_tmptable; j++){
				fprintf(stdout, "%4d ", tmptable[a][j]);
			}
			fprintf(stdout, "\n");
		}
		fprintf(stdout, "\n");
		*/

		if( n_tmptable < 2 )
			continue;

		//try all combinations 
		for(x=0; x<allind; x++){
			if( allcomb[x][n_genome] == 0 )
				continue;

			//initialize dag[][]
			for(a=0; a<=n_tmptable; a++)
				for(j=0; j<=n_tmptable; j++)
					dag[a][j] = 0;

			//construct dag[][]
			for(y=0; y<n_tmptable; y++){
				for(j=y+1; j<n_tmptable; j++){
					a = b = 1;

					if( tmptable[0][y] > tmptable[0][j] ){
						for(c=1; c<n_genome; c++){
							if ( allcomb[x][c] == 1 ){
								if(    tmptable[c][j] >= tmptable[c][0] 
									&& tmptable[c][y] >  tmptable[c][j] 
									&& tmptable[c][y] - tmptable[c][0] <= k ){
									a++;
								}
							} else if ( allcomb[x][c] == -1 ){
								if(    tmptable[c][j] <= tmptable[c][0] 
									&& tmptable[c][y] <  tmptable[c][j] 
									&& tmptable[c][0] - tmptable[c][y] <= k ){
									a++;
								}
							} else {
								fprintf(stderr, "\nallcomb[x][c] == 0 ?\n\n"), fflush(NULL), exit(0);
							}
						}
					} else if( tmptable[0][j] > tmptable[0][y] ) {
						for(c=1; c<n_genome; c++){
							if ( allcomb[x][c] == 1 ){
								if(    tmptable[c][y] >= tmptable[c][0] 
									&& tmptable[c][j] >  tmptable[c][y] 
									&& tmptable[c][j] - tmptable[c][0] <= k ){
									b++;
								}
							} else if ( allcomb[x][c] == -1 ){
								if(    tmptable[c][y] <= tmptable[c][0] 
									&& tmptable[c][j] <  tmptable[c][y] 
									&& tmptable[c][0] - tmptable[c][j] <= k ){
									b++;
								}
							} else {
								fprintf(stderr, "\nallcomb[x][c] == 0 ?\n\n"), fflush(NULL), exit(0);
							}
						}
					} else {
						fprintf(stderr, "\ntmptable[c][y] == tmptable[c][j] ?\n\n"), fflush(NULL), exit(0);
					}

					if( b == n_genome ){
						dag[y][j] = 1;
					}

					if( a == n_genome ){
						dag[j][y] = 1;
					}
				}
			}

			//call Eppstein to output top n paths
			//use_eppstein(dag, x, n_tmptable, tmptable, xchr, xnomename, n_genome, ofname);

			//Output top n non-overlapped paths 
			non_overlap_n(dag, x, n_tmptable, tmptable, xchr, xnomename, n_genome, ofname, tmpedges, n_tmpedges, dist, pi);
		}
		
	}

	return;					  
}



//search for syntenies with genes in same order -- in two directions
//for circurlar genomes
void syn_finder_order_2ways_circular(int **xchr, int **xpos, char **xname, char **xnomename, int n_genome,
					  int *n_gene, int xmax, int k, char *ofname)
{
	int  i, j, comb, x, y; 

	int maxng; //the max gene position on any chr
	int **dag;
	int a, b, c, d, rest;
	
	int ** tmptable; //keep all combinations
	int n_tmptable;  //number of combinations

	ordersyn * curr;

	EDGE_t * tmpedges;
	int    n_tmpedges;
	int *  dist, *pi;

	if( !(tmptable = calloc(n_genome, sizeof(int *))) )
		FPFE1("out of memory for tmptable\n");
	for( i=0; i<n_genome; i++ ){
		if( !(tmptable[i] = calloc(k+1, sizeof(int *))) )
			FPFE1("out of memory for tmptable[]\n");
	}

	if( !(tmpx=malloc(n_genome*sizeof(int))) )
		FPFE1("out of memory for tmpx\n");

	if( !(allindex = malloc( n_genome * sizeof(int))) )
		FPFE1("out of memory for allindex\n");
    for(comb=i=1; i<=n_genome; i++)
		comb *= 3;
	if( !(allcomb=calloc(comb, sizeof(int *))) )
		FPFE1("out of memory for allcomb\n");
	for(i=0; i<comb; i++){
		if( !(allcomb[i] = calloc(n_genome+1,sizeof(int))) )
			FPFE1("out of memory for allcomb[]\n");

		allcomb[i][n_genome] = 1;
	}

	allind = 0;
	recur_comb_3(n_genome, 0);
	comb = allind;

	//remove unnecessary combinations
	for(i=1; i<allind; i++){
		for(j=0; j<i; j++){
			for(d=0; d<n_genome; d++){
				if( allcomb[i][d] != - allcomb[j][d] )
					break;
			}

			if( d == n_genome ){
				allcomb[i][n_genome] = 0;
			}
		}

		b=0;
		for(d=0; d<n_genome; d++){
			if( allcomb[i][d] )
				b++;
		}
		if( b < 2 || b < NUM_GENOME_CUTOFF )
			allcomb[i][n_genome] = 0;
	}	

	//determine maxng;
	for(maxng=-1, i=0; i<n_genome; i++){
		if( n_gene[i] > maxng )
			maxng = n_gene[i];
	}
	if( maxng <= -1 ) //no genes at all?
		return;  

	//allocate space for dag
	if( !(dag = calloc(maxng+1, sizeof(int *))) )
		FPFE1("out of memory for dag\n");
	for(i=0; i<=maxng; i++){
		if( !(dag[i] = calloc(maxng+1, sizeof(int))) )
			FPFE1("out of memory for dag[]\n");
	}

	//allocate space for tmpepppath
	if( !(tmpepppath = calloc(k+10, sizeof(int))) )
		FPFE1("out of memory for tmpepppath\n");

	n_tmpedges = maxng * maxng;
	if( !(tmpedges = calloc(n_tmpedges, sizeof(EDGE_t))) )
		FPFE1("out of memory for tmpedges\n");
	if( !(dist = calloc(k+10, sizeof(int))) )
		FPFE1("out of memory for dist\n");
	if( !(pi = calloc(k+10, sizeof(int))) )
		FPFE1("out of memory for pi\n");
	if( !(spath = calloc(k+10, sizeof(int))) )
		FPFE1("out of memory for spath\n");


	//initialize oresult, otail, omin
	oresult = otail = NULL;
	omin    = 1e200;

	for(j=0; j<NUM_OUTPUT; j++){
		if( !oresult ){
			if( !(oresult = otail = calloc(1, sizeof(ordersyn))) )
				fprintf(stdout, "out of memory for oresult\n"), fflush(NULL),exit(0);

			otail->p = 1e200;
			otail->n = -1; 
			otail->m = -1; 
			otail->prev = NULL;
			otail->next = NULL;
			otail->c = NULL;
			otail->s = NULL;
			if( !(otail->c = calloc(n_genome, sizeof(int))) )
				fprintf(stdout, "out of memory for otail->c\n"), fflush(NULL),exit(0);
			if( !(otail->s = calloc(k+1, sizeof(int))) )
				fprintf(stdout, "out of memory for otail->s\n"), fflush(NULL),exit(0);				
		} else {
			if( !(curr = calloc(1, sizeof(ordersyn))) )
				fprintf(stdout, "out of memory for curr\n"), fflush(NULL),exit(0);

			curr->p = 1e200;
			curr->n = -1; 
			curr->m = -1; 
			curr->prev = NULL;
			curr->next = NULL;
			curr->c = NULL;
			curr->s = NULL;
			if( !(curr->c = calloc(n_genome, sizeof(int))) )
				fprintf(stdout, "out of memory for curr->c\n"), fflush(NULL),exit(0);
			if( !(curr->s = calloc(k+1, sizeof(int))) )
				fprintf(stdout, "out of memory for curr->s\n"), fflush(NULL),exit(0);

			curr->prev = otail;
			otail->next = curr;

			otail = curr;
		}
	}

	//k by k scan
	for(i=0; i<=n_gene[0]; i++){
		if( xchr[0][i] < 0 )
			continue;

		n_tmptable = 0;
		//for(j=i; j<=n_gene[0] && j<=i+k; j++){
		for(j=i; j<=i+k; j++){			
			rest = (j > n_gene[0]) ? (j % n_gene[0]) : j;
			if( xchr[0][rest] < 0 )
				continue;

			tmptable[0][n_tmptable] = rest;
			b = xchr[0][rest]; 

			for(a=1; a<n_genome; a++)
				tmptable[a][n_tmptable] = xpos[b][a];			

			n_tmptable++;
		}

		if( n_tmptable < 2 )
			continue;

		//try all combinations 
		for(x=0; x<allind; x++){
			if( allcomb[x][n_genome] == 0 )
				continue;

			//initialize dag[][]
			for(a=0; a<=n_tmptable; a++)
				for(j=0; j<=n_tmptable; j++)
					dag[a][j] = 0;

			//construct dag[][]
			for(y=0; y<n_tmptable; y++){
				for(j=y+1; j<n_tmptable; j++){
					a = b = 1;

					/*
					if( tmptable[0][y] > tmptable[0][j] ){ //??? possible?
						for(c=1; c<n_genome; c++){
							if ( allcomb[x][c] == 1 ){
								if(    tmptable[c][j] >= tmptable[c][0] 
									&& tmptable[c][y] >  tmptable[c][j] 
									&& tmptable[c][y] - tmptable[c][0] <= k ){
									a++;
								}
							} else if ( allcomb[x][c] == -1 ){
								if(    tmptable[c][j] <= tmptable[c][0] 
									&& tmptable[c][y] <  tmptable[c][j] 
									&& tmptable[c][0] - tmptable[c][y] <= k ){
									a++;
								}
							} else {
								fprintf(stderr, "\nallcomb[x][c] == 0 ?\n\n"), fflush(NULL), exit(0);
							}
						}
					} else 
					*/
					//if( tmptable[0][j] > tmptable[0][y] ) {  //0 => y -> j
						for(c=1; c<n_genome; c++){
							if ( allcomb[x][c] == 1 ){ //0 => y -> j
								//case 1: c0 -> y -> j   
								if(    tmptable[c][y] >= tmptable[c][0] 
									&& tmptable[c][j] >  tmptable[c][y] 
									&& tmptable[c][j] - tmptable[c][0] <= k ){
									b++;
								}

								//case 2: j -> ....-> c0 -> y 
								if( tmptable[c][j] < tmptable[c][0] && tmptable[c][0] <= tmptable[c][y] 
								&& tmptable[c][j] + n_gene[c] - tmptable[c][0] <= k ){
									b++;
								}

								//case 3: y -> j -> ....-> c0
								if( tmptable[c][y] < tmptable[c][j] && tmptable[c][j] < tmptable[c][0] 
								&&  tmptable[c][j] + n_gene[c] - tmptable[c][0] <= k ){
									b ++;
								}
							} else if ( allcomb[x][c] == -1 ){  // j -> y => 0
								//case 1: j -> y => 0
								if(    tmptable[c][y] <= tmptable[c][0] 
									&& tmptable[c][j] <  tmptable[c][y] 
									&& tmptable[c][0] - tmptable[c][j] <= k ){
									b++;
								}

								//case 2: y => 0 -> .....-> j
								if( tmptable[c][y] <= tmptable[c][0]
								&&  tmptable[c][0] < tmptable[c][j]
								&&  tmptable[c][0] + n_gene[c] -tmptable[c][j] <= k ){
									b ++;
								}

								//case 3: 0 -> ... -> j -> y
								if( tmptable[c][0] < tmptable[c][j] && tmptable[c][j] < tmptable[c][y]
								&&  tmptable[c][0] + n_gene[c] -tmptable[c][j] <= k ){
									b ++;
								}
							} else {
								fprintf(stderr, "\nallcomb[x][c] == 0 ?\n\n"), fflush(NULL), exit(0);
							}
						}
					//} else {
					//	fprintf(stderr, "\ntmptable[c][y] == tmptable[c][j] ?\n\n"), fflush(NULL), exit(0);
					//}

					if( b == n_genome ){
						dag[y][j] = 1;
					}

					//if( a == n_genome ){
					//	dag[j][y] = 1;
					//}
				}
			}

			//call Eppstein to output top n paths
			//use_eppstein(dag, x, n_tmptable, tmptable, xchr, xnomename, n_genome, ofname);

			//Output top n non-overlapped paths 
			non_overlap_n(dag, x, n_tmptable, tmptable, xchr, xnomename, n_genome, ofname, tmpedges, n_tmpedges, dist, pi);
		}
		
	}

	return;					  
}



